from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class ListCatalogItemsResponse(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: ListMatchingItemsResponse = self._get_value(ListMatchingItemsResponse, "payload")
        else:
            self.payload: ListMatchingItemsResponse = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class ListMatchingItemsResponse(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "Items" in data:
            self.Items: ItemList = self._get_value(ItemList, "Items")
        else:
            self.Items: ItemList = None


class GetCatalogItemResponse(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Item = self._get_value(Item, "payload")
        else:
            self.payload: Item = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class Item(__BaseDictObject):
    """
    An item in the Amazon catalog.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Identifiers" in data:
            self.Identifiers: IdentifierType = self._get_value(IdentifierType, "Identifiers")
        else:
            self.Identifiers: IdentifierType = None
        if "AttributeSets" in data:
            self.AttributeSets: AttributeSetList = self._get_value(AttributeSetList, "AttributeSets")
        else:
            self.AttributeSets: AttributeSetList = None
        if "Relationships" in data:
            self.Relationships: RelationshipList = self._get_value(RelationshipList, "Relationships")
        else:
            self.Relationships: RelationshipList = None
        if "SalesRankings" in data:
            self.SalesRankings: SalesRankList = self._get_value(SalesRankList, "SalesRankings")
        else:
            self.SalesRankings: SalesRankList = None


class IdentifierType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceASIN" in data:
            self.MarketplaceASIN: ASINIdentifier = self._get_value(ASINIdentifier, "MarketplaceASIN")
        else:
            self.MarketplaceASIN: ASINIdentifier = None
        if "SKUIdentifier" in data:
            self.SKUIdentifier: SellerSKUIdentifier = self._get_value(SellerSKUIdentifier, "SKUIdentifier")
        else:
            self.SKUIdentifier: SellerSKUIdentifier = None


class ASINIdentifier(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None


class SellerSKUIdentifier(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "SellerId" in data:
            self.SellerId: str = self._get_value(str, "SellerId")
        else:
            self.SellerId: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None


class AttributeSetListType(__BaseDictObject):
    """
    The attributes of the item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Actor" in data:
            self.Actor: _List[str] = [str(datum) for datum in data["Actor"]]
        else:
            self.Actor: _List[str] = []
        if "Artist" in data:
            self.Artist: _List[str] = [str(datum) for datum in data["Artist"]]
        else:
            self.Artist: _List[str] = []
        if "AspectRatio" in data:
            self.AspectRatio: str = self._get_value(str, "AspectRatio")
        else:
            self.AspectRatio: str = None
        if "AudienceRating" in data:
            self.AudienceRating: str = self._get_value(str, "AudienceRating")
        else:
            self.AudienceRating: str = None
        if "Author" in data:
            self.Author: _List[str] = [str(datum) for datum in data["Author"]]
        else:
            self.Author: _List[str] = []
        if "BackFinding" in data:
            self.BackFinding: str = self._get_value(str, "BackFinding")
        else:
            self.BackFinding: str = None
        if "BandMaterialType" in data:
            self.BandMaterialType: str = self._get_value(str, "BandMaterialType")
        else:
            self.BandMaterialType: str = None
        if "Binding" in data:
            self.Binding: str = self._get_value(str, "Binding")
        else:
            self.Binding: str = None
        if "BlurayRegion" in data:
            self.BlurayRegion: str = self._get_value(str, "BlurayRegion")
        else:
            self.BlurayRegion: str = None
        if "Brand" in data:
            self.Brand: str = self._get_value(str, "Brand")
        else:
            self.Brand: str = None
        if "CeroAgeRating" in data:
            self.CeroAgeRating: str = self._get_value(str, "CeroAgeRating")
        else:
            self.CeroAgeRating: str = None
        if "ChainType" in data:
            self.ChainType: str = self._get_value(str, "ChainType")
        else:
            self.ChainType: str = None
        if "ClaspType" in data:
            self.ClaspType: str = self._get_value(str, "ClaspType")
        else:
            self.ClaspType: str = None
        if "Color" in data:
            self.Color: str = self._get_value(str, "Color")
        else:
            self.Color: str = None
        if "CpuManufacturer" in data:
            self.CpuManufacturer: str = self._get_value(str, "CpuManufacturer")
        else:
            self.CpuManufacturer: str = None
        if "CpuSpeed" in data:
            self.CpuSpeed: DecimalWithUnits = self._get_value(DecimalWithUnits, "CpuSpeed")
        else:
            self.CpuSpeed: DecimalWithUnits = None
        if "CpuType" in data:
            self.CpuType: str = self._get_value(str, "CpuType")
        else:
            self.CpuType: str = None
        if "Creator" in data:
            self.Creator: _List[CreatorType] = [CreatorType(datum) for datum in data["Creator"]]
        else:
            self.Creator: _List[CreatorType] = []
        if "Department" in data:
            self.Department: str = self._get_value(str, "Department")
        else:
            self.Department: str = None
        if "Director" in data:
            self.Director: _List[str] = [str(datum) for datum in data["Director"]]
        else:
            self.Director: _List[str] = []
        if "DisplaySize" in data:
            self.DisplaySize: DecimalWithUnits = self._get_value(DecimalWithUnits, "DisplaySize")
        else:
            self.DisplaySize: DecimalWithUnits = None
        if "Edition" in data:
            self.Edition: str = self._get_value(str, "Edition")
        else:
            self.Edition: str = None
        if "EpisodeSequence" in data:
            self.EpisodeSequence: str = self._get_value(str, "EpisodeSequence")
        else:
            self.EpisodeSequence: str = None
        if "EsrbAgeRating" in data:
            self.EsrbAgeRating: str = self._get_value(str, "EsrbAgeRating")
        else:
            self.EsrbAgeRating: str = None
        if "Feature" in data:
            self.Feature: _List[str] = [str(datum) for datum in data["Feature"]]
        else:
            self.Feature: _List[str] = []
        if "Flavor" in data:
            self.Flavor: str = self._get_value(str, "Flavor")
        else:
            self.Flavor: str = None
        if "Format" in data:
            self.Format: _List[str] = [str(datum) for datum in data["Format"]]
        else:
            self.Format: _List[str] = []
        if "GemType" in data:
            self.GemType: _List[str] = [str(datum) for datum in data["GemType"]]
        else:
            self.GemType: _List[str] = []
        if "Genre" in data:
            self.Genre: str = self._get_value(str, "Genre")
        else:
            self.Genre: str = None
        if "GolfClubFlex" in data:
            self.GolfClubFlex: str = self._get_value(str, "GolfClubFlex")
        else:
            self.GolfClubFlex: str = None
        if "GolfClubLoft" in data:
            self.GolfClubLoft: DecimalWithUnits = self._get_value(DecimalWithUnits, "GolfClubLoft")
        else:
            self.GolfClubLoft: DecimalWithUnits = None
        if "HandOrientation" in data:
            self.HandOrientation: str = self._get_value(str, "HandOrientation")
        else:
            self.HandOrientation: str = None
        if "HardDiskInterface" in data:
            self.HardDiskInterface: str = self._get_value(str, "HardDiskInterface")
        else:
            self.HardDiskInterface: str = None
        if "HardDiskSize" in data:
            self.HardDiskSize: DecimalWithUnits = self._get_value(DecimalWithUnits, "HardDiskSize")
        else:
            self.HardDiskSize: DecimalWithUnits = None
        if "HardwarePlatform" in data:
            self.HardwarePlatform: str = self._get_value(str, "HardwarePlatform")
        else:
            self.HardwarePlatform: str = None
        if "HazardousMaterialType" in data:
            self.HazardousMaterialType: str = self._get_value(str, "HazardousMaterialType")
        else:
            self.HazardousMaterialType: str = None
        if "ItemDimensions" in data:
            self.ItemDimensions: DimensionType = self._get_value(DimensionType, "ItemDimensions")
        else:
            self.ItemDimensions: DimensionType = None
        if "IsAdultProduct" in data:
            self.IsAdultProduct: bool = self._get_value(convert_bool, "IsAdultProduct")
        else:
            self.IsAdultProduct: bool = None
        if "IsAutographed" in data:
            self.IsAutographed: bool = self._get_value(convert_bool, "IsAutographed")
        else:
            self.IsAutographed: bool = None
        if "IsEligibleForTradeIn" in data:
            self.IsEligibleForTradeIn: bool = self._get_value(convert_bool, "IsEligibleForTradeIn")
        else:
            self.IsEligibleForTradeIn: bool = None
        if "IsMemorabilia" in data:
            self.IsMemorabilia: bool = self._get_value(convert_bool, "IsMemorabilia")
        else:
            self.IsMemorabilia: bool = None
        if "IssuesPerYear" in data:
            self.IssuesPerYear: str = self._get_value(str, "IssuesPerYear")
        else:
            self.IssuesPerYear: str = None
        if "ItemPartNumber" in data:
            self.ItemPartNumber: str = self._get_value(str, "ItemPartNumber")
        else:
            self.ItemPartNumber: str = None
        if "Label" in data:
            self.Label: str = self._get_value(str, "Label")
        else:
            self.Label: str = None
        if "Languages" in data:
            self.Languages: _List[LanguageType] = [LanguageType(datum) for datum in data["Languages"]]
        else:
            self.Languages: _List[LanguageType] = []
        if "LegalDisclaimer" in data:
            self.LegalDisclaimer: str = self._get_value(str, "LegalDisclaimer")
        else:
            self.LegalDisclaimer: str = None
        if "ListPrice" in data:
            self.ListPrice: Price = self._get_value(Price, "ListPrice")
        else:
            self.ListPrice: Price = None
        if "Manufacturer" in data:
            self.Manufacturer: str = self._get_value(str, "Manufacturer")
        else:
            self.Manufacturer: str = None
        if "ManufacturerMaximumAge" in data:
            self.ManufacturerMaximumAge: DecimalWithUnits = self._get_value(DecimalWithUnits, "ManufacturerMaximumAge")
        else:
            self.ManufacturerMaximumAge: DecimalWithUnits = None
        if "ManufacturerMinimumAge" in data:
            self.ManufacturerMinimumAge: DecimalWithUnits = self._get_value(DecimalWithUnits, "ManufacturerMinimumAge")
        else:
            self.ManufacturerMinimumAge: DecimalWithUnits = None
        if "ManufacturerPartsWarrantyDescription" in data:
            self.ManufacturerPartsWarrantyDescription: str = self._get_value(
                str, "ManufacturerPartsWarrantyDescription"
            )
        else:
            self.ManufacturerPartsWarrantyDescription: str = None
        if "MaterialType" in data:
            self.MaterialType: _List[str] = [str(datum) for datum in data["MaterialType"]]
        else:
            self.MaterialType: _List[str] = []
        if "MaximumResolution" in data:
            self.MaximumResolution: DecimalWithUnits = self._get_value(DecimalWithUnits, "MaximumResolution")
        else:
            self.MaximumResolution: DecimalWithUnits = None
        if "MediaType" in data:
            self.MediaType: _List[str] = [str(datum) for datum in data["MediaType"]]
        else:
            self.MediaType: _List[str] = []
        if "MetalStamp" in data:
            self.MetalStamp: str = self._get_value(str, "MetalStamp")
        else:
            self.MetalStamp: str = None
        if "MetalType" in data:
            self.MetalType: str = self._get_value(str, "MetalType")
        else:
            self.MetalType: str = None
        if "Model" in data:
            self.Model: str = self._get_value(str, "Model")
        else:
            self.Model: str = None
        if "NumberOfDiscs" in data:
            self.NumberOfDiscs: int = self._get_value(int, "NumberOfDiscs")
        else:
            self.NumberOfDiscs: int = None
        if "NumberOfIssues" in data:
            self.NumberOfIssues: int = self._get_value(int, "NumberOfIssues")
        else:
            self.NumberOfIssues: int = None
        if "NumberOfItems" in data:
            self.NumberOfItems: int = self._get_value(int, "NumberOfItems")
        else:
            self.NumberOfItems: int = None
        if "NumberOfPages" in data:
            self.NumberOfPages: int = self._get_value(int, "NumberOfPages")
        else:
            self.NumberOfPages: int = None
        if "NumberOfTracks" in data:
            self.NumberOfTracks: int = self._get_value(int, "NumberOfTracks")
        else:
            self.NumberOfTracks: int = None
        if "OperatingSystem" in data:
            self.OperatingSystem: _List[str] = [str(datum) for datum in data["OperatingSystem"]]
        else:
            self.OperatingSystem: _List[str] = []
        if "OpticalZoom" in data:
            self.OpticalZoom: DecimalWithUnits = self._get_value(DecimalWithUnits, "OpticalZoom")
        else:
            self.OpticalZoom: DecimalWithUnits = None
        if "PackageDimensions" in data:
            self.PackageDimensions: DimensionType = self._get_value(DimensionType, "PackageDimensions")
        else:
            self.PackageDimensions: DimensionType = None
        if "PackageQuantity" in data:
            self.PackageQuantity: int = self._get_value(int, "PackageQuantity")
        else:
            self.PackageQuantity: int = None
        if "PartNumber" in data:
            self.PartNumber: str = self._get_value(str, "PartNumber")
        else:
            self.PartNumber: str = None
        if "PegiRating" in data:
            self.PegiRating: str = self._get_value(str, "PegiRating")
        else:
            self.PegiRating: str = None
        if "Platform" in data:
            self.Platform: _List[str] = [str(datum) for datum in data["Platform"]]
        else:
            self.Platform: _List[str] = []
        if "ProcessorCount" in data:
            self.ProcessorCount: int = self._get_value(int, "ProcessorCount")
        else:
            self.ProcessorCount: int = None
        if "ProductGroup" in data:
            self.ProductGroup: str = self._get_value(str, "ProductGroup")
        else:
            self.ProductGroup: str = None
        if "ProductTypeName" in data:
            self.ProductTypeName: str = self._get_value(str, "ProductTypeName")
        else:
            self.ProductTypeName: str = None
        if "ProductTypeSubcategory" in data:
            self.ProductTypeSubcategory: str = self._get_value(str, "ProductTypeSubcategory")
        else:
            self.ProductTypeSubcategory: str = None
        if "PublicationDate" in data:
            self.PublicationDate: str = self._get_value(str, "PublicationDate")
        else:
            self.PublicationDate: str = None
        if "Publisher" in data:
            self.Publisher: str = self._get_value(str, "Publisher")
        else:
            self.Publisher: str = None
        if "RegionCode" in data:
            self.RegionCode: str = self._get_value(str, "RegionCode")
        else:
            self.RegionCode: str = None
        if "ReleaseDate" in data:
            self.ReleaseDate: str = self._get_value(str, "ReleaseDate")
        else:
            self.ReleaseDate: str = None
        if "RingSize" in data:
            self.RingSize: str = self._get_value(str, "RingSize")
        else:
            self.RingSize: str = None
        if "RunningTime" in data:
            self.RunningTime: DecimalWithUnits = self._get_value(DecimalWithUnits, "RunningTime")
        else:
            self.RunningTime: DecimalWithUnits = None
        if "ShaftMaterial" in data:
            self.ShaftMaterial: str = self._get_value(str, "ShaftMaterial")
        else:
            self.ShaftMaterial: str = None
        if "Scent" in data:
            self.Scent: str = self._get_value(str, "Scent")
        else:
            self.Scent: str = None
        if "SeasonSequence" in data:
            self.SeasonSequence: str = self._get_value(str, "SeasonSequence")
        else:
            self.SeasonSequence: str = None
        if "SeikodoProductCode" in data:
            self.SeikodoProductCode: str = self._get_value(str, "SeikodoProductCode")
        else:
            self.SeikodoProductCode: str = None
        if "Size" in data:
            self.Size: str = self._get_value(str, "Size")
        else:
            self.Size: str = None
        if "SizePerPearl" in data:
            self.SizePerPearl: str = self._get_value(str, "SizePerPearl")
        else:
            self.SizePerPearl: str = None
        if "SmallImage" in data:
            self.SmallImage: Image = self._get_value(Image, "SmallImage")
        else:
            self.SmallImage: Image = None
        if "Studio" in data:
            self.Studio: str = self._get_value(str, "Studio")
        else:
            self.Studio: str = None
        if "SubscriptionLength" in data:
            self.SubscriptionLength: DecimalWithUnits = self._get_value(DecimalWithUnits, "SubscriptionLength")
        else:
            self.SubscriptionLength: DecimalWithUnits = None
        if "SystemMemorySize" in data:
            self.SystemMemorySize: DecimalWithUnits = self._get_value(DecimalWithUnits, "SystemMemorySize")
        else:
            self.SystemMemorySize: DecimalWithUnits = None
        if "SystemMemoryType" in data:
            self.SystemMemoryType: str = self._get_value(str, "SystemMemoryType")
        else:
            self.SystemMemoryType: str = None
        if "TheatricalReleaseDate" in data:
            self.TheatricalReleaseDate: str = self._get_value(str, "TheatricalReleaseDate")
        else:
            self.TheatricalReleaseDate: str = None
        if "Title" in data:
            self.Title: str = self._get_value(str, "Title")
        else:
            self.Title: str = None
        if "TotalDiamondWeight" in data:
            self.TotalDiamondWeight: DecimalWithUnits = self._get_value(DecimalWithUnits, "TotalDiamondWeight")
        else:
            self.TotalDiamondWeight: DecimalWithUnits = None
        if "TotalGemWeight" in data:
            self.TotalGemWeight: DecimalWithUnits = self._get_value(DecimalWithUnits, "TotalGemWeight")
        else:
            self.TotalGemWeight: DecimalWithUnits = None
        if "Warranty" in data:
            self.Warranty: str = self._get_value(str, "Warranty")
        else:
            self.Warranty: str = None
        if "WeeeTaxValue" in data:
            self.WeeeTaxValue: Price = self._get_value(Price, "WeeeTaxValue")
        else:
            self.WeeeTaxValue: Price = None


class DecimalWithUnits(__BaseDictObject):
    """
    The decimal value and unit.
    """

    def __init__(self, data):
        super().__init__(data)
        if "value" in data:
            self.value: float = self._get_value(float, "value")
        else:
            self.value: float = None
        if "Units" in data:
            self.Units: str = self._get_value(str, "Units")
        else:
            self.Units: str = None


class CreatorType(__BaseDictObject):
    """
    The creator type attribute of an item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "value" in data:
            self.value: str = self._get_value(str, "value")
        else:
            self.value: str = None
        if "Role" in data:
            self.Role: str = self._get_value(str, "Role")
        else:
            self.Role: str = None


class DimensionType(__BaseDictObject):
    """
    The dimension type attribute of an item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Height" in data:
            self.Height: DecimalWithUnits = self._get_value(DecimalWithUnits, "Height")
        else:
            self.Height: DecimalWithUnits = None
        if "Length" in data:
            self.Length: DecimalWithUnits = self._get_value(DecimalWithUnits, "Length")
        else:
            self.Length: DecimalWithUnits = None
        if "Width" in data:
            self.Width: DecimalWithUnits = self._get_value(DecimalWithUnits, "Width")
        else:
            self.Width: DecimalWithUnits = None
        if "Weight" in data:
            self.Weight: DecimalWithUnits = self._get_value(DecimalWithUnits, "Weight")
        else:
            self.Weight: DecimalWithUnits = None


class LanguageType(__BaseDictObject):
    """
    The language type attribute of an item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Name" in data:
            self.Name: str = self._get_value(str, "Name")
        else:
            self.Name: str = None
        if "Type" in data:
            self.Type: str = self._get_value(str, "Type")
        else:
            self.Type: str = None
        if "AudioFormat" in data:
            self.AudioFormat: str = self._get_value(str, "AudioFormat")
        else:
            self.AudioFormat: str = None


class Image(__BaseDictObject):
    """
    The image attribute of the item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "URL" in data:
            self.URL: str = self._get_value(str, "URL")
        else:
            self.URL: str = None
        if "Height" in data:
            self.Height: DecimalWithUnits = self._get_value(DecimalWithUnits, "Height")
        else:
            self.Height: DecimalWithUnits = None
        if "Width" in data:
            self.Width: DecimalWithUnits = self._get_value(DecimalWithUnits, "Width")
        else:
            self.Width: DecimalWithUnits = None


class Price(__BaseDictObject):
    """
    The price attribute of the item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Amount" in data:
            self.Amount: float = self._get_value(float, "Amount")
        else:
            self.Amount: float = None
        if "CurrencyCode" in data:
            self.CurrencyCode: str = self._get_value(str, "CurrencyCode")
        else:
            self.CurrencyCode: str = None


class RelationshipType(__BaseDictObject):
    """
    Specific variations of the item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Identifiers" in data:
            self.Identifiers: IdentifierType = self._get_value(IdentifierType, "Identifiers")
        else:
            self.Identifiers: IdentifierType = None
        if "Color" in data:
            self.Color: str = self._get_value(str, "Color")
        else:
            self.Color: str = None
        if "Edition" in data:
            self.Edition: str = self._get_value(str, "Edition")
        else:
            self.Edition: str = None
        if "Flavor" in data:
            self.Flavor: str = self._get_value(str, "Flavor")
        else:
            self.Flavor: str = None
        if "GemType" in data:
            self.GemType: _List[str] = [str(datum) for datum in data["GemType"]]
        else:
            self.GemType: _List[str] = []
        if "GolfClubFlex" in data:
            self.GolfClubFlex: str = self._get_value(str, "GolfClubFlex")
        else:
            self.GolfClubFlex: str = None
        if "HandOrientation" in data:
            self.HandOrientation: str = self._get_value(str, "HandOrientation")
        else:
            self.HandOrientation: str = None
        if "HardwarePlatform" in data:
            self.HardwarePlatform: str = self._get_value(str, "HardwarePlatform")
        else:
            self.HardwarePlatform: str = None
        if "MaterialType" in data:
            self.MaterialType: _List[str] = [str(datum) for datum in data["MaterialType"]]
        else:
            self.MaterialType: _List[str] = []
        if "MetalType" in data:
            self.MetalType: str = self._get_value(str, "MetalType")
        else:
            self.MetalType: str = None
        if "Model" in data:
            self.Model: str = self._get_value(str, "Model")
        else:
            self.Model: str = None
        if "OperatingSystem" in data:
            self.OperatingSystem: _List[str] = [str(datum) for datum in data["OperatingSystem"]]
        else:
            self.OperatingSystem: _List[str] = []
        if "ProductTypeSubcategory" in data:
            self.ProductTypeSubcategory: str = self._get_value(str, "ProductTypeSubcategory")
        else:
            self.ProductTypeSubcategory: str = None
        if "RingSize" in data:
            self.RingSize: str = self._get_value(str, "RingSize")
        else:
            self.RingSize: str = None
        if "ShaftMaterial" in data:
            self.ShaftMaterial: str = self._get_value(str, "ShaftMaterial")
        else:
            self.ShaftMaterial: str = None
        if "Scent" in data:
            self.Scent: str = self._get_value(str, "Scent")
        else:
            self.Scent: str = None
        if "Size" in data:
            self.Size: str = self._get_value(str, "Size")
        else:
            self.Size: str = None
        if "SizePerPearl" in data:
            self.SizePerPearl: str = self._get_value(str, "SizePerPearl")
        else:
            self.SizePerPearl: str = None
        if "GolfClubLoft" in data:
            self.GolfClubLoft: DecimalWithUnits = self._get_value(DecimalWithUnits, "GolfClubLoft")
        else:
            self.GolfClubLoft: DecimalWithUnits = None
        if "TotalDiamondWeight" in data:
            self.TotalDiamondWeight: DecimalWithUnits = self._get_value(DecimalWithUnits, "TotalDiamondWeight")
        else:
            self.TotalDiamondWeight: DecimalWithUnits = None
        if "TotalGemWeight" in data:
            self.TotalGemWeight: DecimalWithUnits = self._get_value(DecimalWithUnits, "TotalGemWeight")
        else:
            self.TotalGemWeight: DecimalWithUnits = None
        if "PackageQuantity" in data:
            self.PackageQuantity: int = self._get_value(int, "PackageQuantity")
        else:
            self.PackageQuantity: int = None
        if "ItemDimensions" in data:
            self.ItemDimensions: DimensionType = self._get_value(DimensionType, "ItemDimensions")
        else:
            self.ItemDimensions: DimensionType = None


class SalesRankType(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "ProductCategoryId" in data:
            self.ProductCategoryId: str = self._get_value(str, "ProductCategoryId")
        else:
            self.ProductCategoryId: str = None
        if "Rank" in data:
            self.Rank: int = self._get_value(int, "Rank")
        else:
            self.Rank: int = None


class ListCatalogCategoriesResponse(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: ListOfCategories = self._get_value(ListOfCategories, "payload")
        else:
            self.payload: ListOfCategories = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class Categories(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "ProductCategoryId" in data:
            self.ProductCategoryId: str = self._get_value(str, "ProductCategoryId")
        else:
            self.ProductCategoryId: str = None
        if "ProductCategoryName" in data:
            self.ProductCategoryName: str = self._get_value(str, "ProductCategoryName")
        else:
            self.ProductCategoryName: str = None
        if "parent" in data:
            self.parent: dict = self._get_value(dict, "parent")
        else:
            self.parent: dict = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ItemList(list, _List["Item"]):
    """
    A list of items.
    """

    def __init__(self, data):
        super().__init__([Item(datum) for datum in data])
        self.data = data


class AttributeSetList(list, _List["AttributeSetListType"]):
    """
    A list of attributes for the item.
    """

    def __init__(self, data):
        super().__init__([AttributeSetListType(datum) for datum in data])
        self.data = data


class RelationshipList(list, _List["RelationshipType"]):
    """
    A list of variation relationship information, if applicable for the item.
    """

    def __init__(self, data):
        super().__init__([RelationshipType(datum) for datum in data])
        self.data = data


class SalesRankList(list, _List["SalesRankType"]):
    """
    A list of sales rank information for the item by category.
    """

    def __init__(self, data):
        super().__init__([SalesRankType(datum) for datum in data])
        self.data = data


class ListOfCategories(list, _List["Categories"]):
    """ """

    def __init__(self, data):
        super().__init__([Categories(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class CatalogItemsV0Client(__BaseClient):
    def listCatalogItems(
        self,
        MarketplaceId: str,
        Query: str = None,
        QueryContextId: str = None,
        SellerSKU: str = None,
        UPC: str = None,
        EAN: str = None,
        ISBN: str = None,
        JAN: str = None,
    ):
        """
                Returns a list of items and their attributes, based on a search query or item identifiers that you specify. When based on a search query, provide the Query parameter and optionally, the QueryContextId parameter. When based on item identifiers, provide a single appropriate parameter based on the identifier type, and specify the associated item value.
        MarketplaceId is always required. At least one of Query, SellerSKU, UPC, EAN, ISBN, JAN is also required.
        This operation returns a maximum of ten products and does not return non-buyable products.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 6 | 40 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/catalog/v0/items"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        if Query is not None:
            params["Query"] = Query
        if QueryContextId is not None:
            params["QueryContextId"] = QueryContextId
        if SellerSKU is not None:
            params["SellerSKU"] = SellerSKU
        if UPC is not None:
            params["UPC"] = UPC
        if EAN is not None:
            params["EAN"] = EAN
        if ISBN is not None:
            params["ISBN"] = ISBN
        if JAN is not None:
            params["JAN"] = JAN
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListCatalogItemsResponse,
            400: ListCatalogItemsResponse,
            401: ListCatalogItemsResponse,
            403: ListCatalogItemsResponse,
            404: ListCatalogItemsResponse,
            429: ListCatalogItemsResponse,
            500: ListCatalogItemsResponse,
            503: ListCatalogItemsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getCatalogItem(
        self,
        asin: str,
        MarketplaceId: str,
    ):
        """
                Returns a specified item and its attributes.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 2 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/catalog/v0/items/{asin}"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetCatalogItemResponse,
            400: GetCatalogItemResponse,
            401: GetCatalogItemResponse,
            403: GetCatalogItemResponse,
            404: GetCatalogItemResponse,
            429: GetCatalogItemResponse,
            500: GetCatalogItemResponse,
            503: GetCatalogItemResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def listCatalogCategories(
        self,
        MarketplaceId: str,
        ASIN: str = None,
        SellerSKU: str = None,
    ):
        """
                Returns the parent categories to which an item belongs, based on the specified ASIN or SellerSKU.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 1 | 40 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/catalog/v0/categories"
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = MarketplaceId
        if ASIN is not None:
            params["ASIN"] = ASIN
        if SellerSKU is not None:
            params["SellerSKU"] = SellerSKU
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ListCatalogCategoriesResponse,
            400: ListCatalogCategoriesResponse,
            401: ListCatalogCategoriesResponse,
            403: ListCatalogCategoriesResponse,
            404: ListCatalogCategoriesResponse,
            429: ListCatalogCategoriesResponse,
            500: ListCatalogCategoriesResponse,
            503: ListCatalogCategoriesResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class Report(__BaseDictObject):
    """
    Detailed information about the report.
    """

    def __init__(self, data):
        super().__init__(data)
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []
        if "reportId" in data:
            self.reportId: str = self._get_value(str, "reportId")
        else:
            self.reportId: str = None
        if "reportType" in data:
            self.reportType: str = self._get_value(str, "reportType")
        else:
            self.reportType: str = None
        if "dataStartTime" in data:
            self.dataStartTime: str = self._get_value(str, "dataStartTime")
        else:
            self.dataStartTime: str = None
        if "dataEndTime" in data:
            self.dataEndTime: str = self._get_value(str, "dataEndTime")
        else:
            self.dataEndTime: str = None
        if "reportScheduleId" in data:
            self.reportScheduleId: str = self._get_value(str, "reportScheduleId")
        else:
            self.reportScheduleId: str = None
        if "createdTime" in data:
            self.createdTime: str = self._get_value(str, "createdTime")
        else:
            self.createdTime: str = None
        if "processingStatus" in data:
            self.processingStatus: str = self._get_value(str, "processingStatus")
        else:
            self.processingStatus: str = None
        if "processingStartTime" in data:
            self.processingStartTime: str = self._get_value(str, "processingStartTime")
        else:
            self.processingStartTime: str = None
        if "processingEndTime" in data:
            self.processingEndTime: str = self._get_value(str, "processingEndTime")
        else:
            self.processingEndTime: str = None
        if "reportDocumentId" in data:
            self.reportDocumentId: str = self._get_value(str, "reportDocumentId")
        else:
            self.reportDocumentId: str = None


class CreateReportScheduleSpecification(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "reportType" in data:
            self.reportType: str = self._get_value(str, "reportType")
        else:
            self.reportType: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []
        if "reportOptions" in data:
            self.reportOptions: ReportOptions = self._get_value(ReportOptions, "reportOptions")
        else:
            self.reportOptions: ReportOptions = None
        if "period" in data:
            self.period: str = self._get_value(str, "period")
        else:
            self.period: str = None
        if "nextReportCreationTime" in data:
            self.nextReportCreationTime: str = self._get_value(str, "nextReportCreationTime")
        else:
            self.nextReportCreationTime: str = None


class CreateReportSpecification(__BaseDictObject):
    """
    Information required to create the report.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reportOptions" in data:
            self.reportOptions: ReportOptions = self._get_value(ReportOptions, "reportOptions")
        else:
            self.reportOptions: ReportOptions = None
        if "reportType" in data:
            self.reportType: str = self._get_value(str, "reportType")
        else:
            self.reportType: str = None
        if "dataStartTime" in data:
            self.dataStartTime: str = self._get_value(str, "dataStartTime")
        else:
            self.dataStartTime: str = None
        if "dataEndTime" in data:
            self.dataEndTime: str = self._get_value(str, "dataEndTime")
        else:
            self.dataEndTime: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []


class ReportOptions(__BaseDictObject):
    """
    Additional information passed to reports. This varies by report type.
    """

    def __init__(self, data):
        super().__init__(data)


class ReportSchedule(__BaseDictObject):
    """
    Detailed information about a report schedule.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reportScheduleId" in data:
            self.reportScheduleId: str = self._get_value(str, "reportScheduleId")
        else:
            self.reportScheduleId: str = None
        if "reportType" in data:
            self.reportType: str = self._get_value(str, "reportType")
        else:
            self.reportType: str = None
        if "marketplaceIds" in data:
            self.marketplaceIds: _List[str] = [str(datum) for datum in data["marketplaceIds"]]
        else:
            self.marketplaceIds: _List[str] = []
        if "reportOptions" in data:
            self.reportOptions: ReportOptions = self._get_value(ReportOptions, "reportOptions")
        else:
            self.reportOptions: ReportOptions = None
        if "period" in data:
            self.period: str = self._get_value(str, "period")
        else:
            self.period: str = None
        if "nextReportCreationTime" in data:
            self.nextReportCreationTime: str = self._get_value(str, "nextReportCreationTime")
        else:
            self.nextReportCreationTime: str = None


class ReportScheduleList(__BaseDictObject):
    """
    A list of report schedules.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reportSchedules" in data:
            self.reportSchedules: _List[ReportSchedule] = [ReportSchedule(datum) for datum in data["reportSchedules"]]
        else:
            self.reportSchedules: _List[ReportSchedule] = []


class CreateReportResponse(__BaseDictObject):
    """
    Response schema.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reportId" in data:
            self.reportId: str = self._get_value(str, "reportId")
        else:
            self.reportId: str = None


class GetReportsResponse(__BaseDictObject):
    """
    The response for the getReports operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reports" in data:
            self.reports: ReportList = self._get_value(ReportList, "reports")
        else:
            self.reports: ReportList = None
        if "nextToken" in data:
            self.nextToken: str = self._get_value(str, "nextToken")
        else:
            self.nextToken: str = None


class CreateReportScheduleResponse(__BaseDictObject):
    """
    Response schema.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reportScheduleId" in data:
            self.reportScheduleId: str = self._get_value(str, "reportScheduleId")
        else:
            self.reportScheduleId: str = None


class ReportDocument(__BaseDictObject):
    """
    Information required for the report document.
    """

    def __init__(self, data):
        super().__init__(data)
        if "reportDocumentId" in data:
            self.reportDocumentId: str = self._get_value(str, "reportDocumentId")
        else:
            self.reportDocumentId: str = None
        if "url" in data:
            self.url: str = self._get_value(str, "url")
        else:
            self.url: str = None
        if "compressionAlgorithm" in data:
            self.compressionAlgorithm: str = self._get_value(str, "compressionAlgorithm")
        else:
            self.compressionAlgorithm: str = None


class ReportList(list, _List["Report"]):
    """
    A list of reports.
    """

    def __init__(self, data):
        super().__init__([Report(datum) for datum in data])
        self.data = data


class Reports20210630Client(__BaseClient):
    def getReports(
        self,
        reportTypes: _List[str] = None,
        processingStatuses: _List[str] = None,
        marketplaceIds: _List[str] = None,
        pageSize: int = None,
        createdSince: str = None,
        createdUntil: str = None,
        nextToken: str = None,
    ):
        """
                Returns report details for the reports that match the filters that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/reports"
        params = {}
        if reportTypes is not None:
            params["reportTypes"] = ",".join(map(str, reportTypes))
        if processingStatuses is not None:
            params["processingStatuses"] = ",".join(map(str, processingStatuses))
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if pageSize is not None:
            params["pageSize"] = pageSize
        if createdSince is not None:
            params["createdSince"] = createdSince
        if createdUntil is not None:
            params["createdUntil"] = createdUntil
        if nextToken is not None:
            params["nextToken"] = nextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetReportsResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createReport(
        self,
        data: CreateReportSpecification,
    ):
        """
                Creates a report.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0167 | 15 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/reports"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            202: CreateReportResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getReport(
        self,
        reportId: str,
    ):
        """
                Returns report details (including the reportDocumentId, if available) for the report that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 2.0 | 15 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/reports/{reportId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: Report,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def cancelReport(
        self,
        reportId: str,
    ):
        """
                Cancels the report that you specify. Only reports with processingStatus=IN_QUEUE can be cancelled. Cancelled reports are returned in subsequent calls to the getReport and getReports operations.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/reports/{reportId}"
        params = {}
        response = self.request(
            path=url,
            method="DELETE",
            params=params,
        )
        response_type = {
            200: None,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getReportSchedules(
        self,
        reportTypes: _List[str],
    ):
        """
                Returns report schedule details that match the filters that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/schedules"
        params = {}
        if reportTypes is not None:
            params["reportTypes"] = ",".join(map(str, reportTypes))
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ReportScheduleList,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def createReportSchedule(
        self,
        data: CreateReportScheduleSpecification,
    ):
        """
                Creates a report schedule. If a report schedule with the same report type and marketplace IDs already exists, it will be cancelled and replaced with this one.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/schedules"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            201: CreateReportScheduleResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getReportSchedule(
        self,
        reportScheduleId: str,
    ):
        """
                Returns report schedule details for the report schedule that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/schedules/{reportScheduleId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ReportSchedule,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def cancelReportSchedule(
        self,
        reportScheduleId: str,
    ):
        """
                Cancels the report schedule that you specify.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0222 | 10 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/schedules/{reportScheduleId}"
        params = {}
        response = self.request(
            path=url,
            method="DELETE",
            params=params,
        )
        response_type = {
            200: None,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getReportDocument(
        self,
        reportDocumentId: str,
    ):
        """
                Returns the information required for retrieving a report document's contents.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 0.0167 | 15 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/reports/2021-06-30/documents/{reportDocumentId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: ReportDocument,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class SubmitInvoicesResponse(__BaseDictObject):
    """
    The response schema for the submitInvoices operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: TransactionId = self._get_value(TransactionId, "payload")
        else:
            self.payload: TransactionId = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class TransactionId(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class SubmitInvoicesRequest(__BaseDictObject):
    """
    The request schema for the submitInvoices operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "invoices" in data:
            self.invoices: _List[Invoice] = [Invoice(datum) for datum in data["invoices"]]
        else:
            self.invoices: _List[Invoice] = []


class Invoice(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "invoiceType" in data:
            self.invoiceType: str = self._get_value(str, "invoiceType")
        else:
            self.invoiceType: str = None
        if "id" in data:
            self.id: str = self._get_value(str, "id")
        else:
            self.id: str = None
        if "referenceNumber" in data:
            self.referenceNumber: str = self._get_value(str, "referenceNumber")
        else:
            self.referenceNumber: str = None
        if "date" in data:
            self.date: DateTime = self._get_value(DateTime, "date")
        else:
            self.date: DateTime = None
        if "remitToParty" in data:
            self.remitToParty: PartyIdentification = self._get_value(PartyIdentification, "remitToParty")
        else:
            self.remitToParty: PartyIdentification = None
        if "shipToParty" in data:
            self.shipToParty: PartyIdentification = self._get_value(PartyIdentification, "shipToParty")
        else:
            self.shipToParty: PartyIdentification = None
        if "shipFromParty" in data:
            self.shipFromParty: PartyIdentification = self._get_value(PartyIdentification, "shipFromParty")
        else:
            self.shipFromParty: PartyIdentification = None
        if "billToParty" in data:
            self.billToParty: PartyIdentification = self._get_value(PartyIdentification, "billToParty")
        else:
            self.billToParty: PartyIdentification = None
        if "paymentTerms" in data:
            self.paymentTerms: PaymentTerms = self._get_value(PaymentTerms, "paymentTerms")
        else:
            self.paymentTerms: PaymentTerms = None
        if "invoiceTotal" in data:
            self.invoiceTotal: Money = self._get_value(Money, "invoiceTotal")
        else:
            self.invoiceTotal: Money = None
        if "taxDetails" in data:
            self.taxDetails: _List[TaxDetails] = [TaxDetails(datum) for datum in data["taxDetails"]]
        else:
            self.taxDetails: _List[TaxDetails] = []
        if "additionalDetails" in data:
            self.additionalDetails: _List[AdditionalDetails] = [
                AdditionalDetails(datum) for datum in data["additionalDetails"]
            ]
        else:
            self.additionalDetails: _List[AdditionalDetails] = []
        if "chargeDetails" in data:
            self.chargeDetails: _List[ChargeDetails] = [ChargeDetails(datum) for datum in data["chargeDetails"]]
        else:
            self.chargeDetails: _List[ChargeDetails] = []
        if "allowanceDetails" in data:
            self.allowanceDetails: _List[AllowanceDetails] = [
                AllowanceDetails(datum) for datum in data["allowanceDetails"]
            ]
        else:
            self.allowanceDetails: _List[AllowanceDetails] = []
        if "items" in data:
            self.items: _List[InvoiceItem] = [InvoiceItem(datum) for datum in data["items"]]
        else:
            self.items: _List[InvoiceItem] = []


class PartyIdentification(__BaseDictObject):
    """ """

    def __init__(self, data):
        super().__init__(data)
        if "partyId" in data:
            self.partyId: str = self._get_value(str, "partyId")
        else:
            self.partyId: str = None
        if "address" in data:
            self.address: Address = self._get_value(Address, "address")
        else:
            self.address: Address = None
        if "taxRegistrationDetails" in data:
            self.taxRegistrationDetails: _List[TaxRegistrationDetails] = [
                TaxRegistrationDetails(datum) for datum in data["taxRegistrationDetails"]
            ]
        else:
            self.taxRegistrationDetails: _List[TaxRegistrationDetails] = []


class TaxRegistrationDetails(__BaseDictObject):
    """
    Tax registration details of the entity.
    """

    def __init__(self, data):
        super().__init__(data)
        if "taxRegistrationType" in data:
            self.taxRegistrationType: str = self._get_value(str, "taxRegistrationType")
        else:
            self.taxRegistrationType: str = None
        if "taxRegistrationNumber" in data:
            self.taxRegistrationNumber: str = self._get_value(str, "taxRegistrationNumber")
        else:
            self.taxRegistrationNumber: str = None


class Address(__BaseDictObject):
    """
    A physical address.
    """

    def __init__(self, data):
        super().__init__(data)
        if "name" in data:
            self.name: str = self._get_value(str, "name")
        else:
            self.name: str = None
        if "addressLine1" in data:
            self.addressLine1: str = self._get_value(str, "addressLine1")
        else:
            self.addressLine1: str = None
        if "addressLine2" in data:
            self.addressLine2: str = self._get_value(str, "addressLine2")
        else:
            self.addressLine2: str = None
        if "addressLine3" in data:
            self.addressLine3: str = self._get_value(str, "addressLine3")
        else:
            self.addressLine3: str = None
        if "city" in data:
            self.city: str = self._get_value(str, "city")
        else:
            self.city: str = None
        if "county" in data:
            self.county: str = self._get_value(str, "county")
        else:
            self.county: str = None
        if "district" in data:
            self.district: str = self._get_value(str, "district")
        else:
            self.district: str = None
        if "stateOrRegion" in data:
            self.stateOrRegion: str = self._get_value(str, "stateOrRegion")
        else:
            self.stateOrRegion: str = None
        if "postalOrZipCode" in data:
            self.postalOrZipCode: str = self._get_value(str, "postalOrZipCode")
        else:
            self.postalOrZipCode: str = None
        if "countryCode" in data:
            self.countryCode: str = self._get_value(str, "countryCode")
        else:
            self.countryCode: str = None
        if "phone" in data:
            self.phone: str = self._get_value(str, "phone")
        else:
            self.phone: str = None


class InvoiceItem(__BaseDictObject):
    """
    Details of the item being invoiced.
    """

    def __init__(self, data):
        super().__init__(data)
        if "itemSequenceNumber" in data:
            self.itemSequenceNumber: int = self._get_value(int, "itemSequenceNumber")
        else:
            self.itemSequenceNumber: int = None
        if "amazonProductIdentifier" in data:
            self.amazonProductIdentifier: str = self._get_value(str, "amazonProductIdentifier")
        else:
            self.amazonProductIdentifier: str = None
        if "vendorProductIdentifier" in data:
            self.vendorProductIdentifier: str = self._get_value(str, "vendorProductIdentifier")
        else:
            self.vendorProductIdentifier: str = None
        if "invoicedQuantity" in data:
            self.invoicedQuantity: ItemQuantity = self._get_value(ItemQuantity, "invoicedQuantity")
        else:
            self.invoicedQuantity: ItemQuantity = None
        if "netCost" in data:
            self.netCost: Money = self._get_value(Money, "netCost")
        else:
            self.netCost: Money = None
        if "purchaseOrderNumber" in data:
            self.purchaseOrderNumber: str = self._get_value(str, "purchaseOrderNumber")
        else:
            self.purchaseOrderNumber: str = None
        if "hsnCode" in data:
            self.hsnCode: str = self._get_value(str, "hsnCode")
        else:
            self.hsnCode: str = None
        if "creditNoteDetails" in data:
            self.creditNoteDetails: CreditNoteDetails = self._get_value(CreditNoteDetails, "creditNoteDetails")
        else:
            self.creditNoteDetails: CreditNoteDetails = None
        if "taxDetails" in data:
            self.taxDetails: _List[TaxDetails] = [TaxDetails(datum) for datum in data["taxDetails"]]
        else:
            self.taxDetails: _List[TaxDetails] = []
        if "chargeDetails" in data:
            self.chargeDetails: _List[ChargeDetails] = [ChargeDetails(datum) for datum in data["chargeDetails"]]
        else:
            self.chargeDetails: _List[ChargeDetails] = []
        if "allowanceDetails" in data:
            self.allowanceDetails: _List[AllowanceDetails] = [
                AllowanceDetails(datum) for datum in data["allowanceDetails"]
            ]
        else:
            self.allowanceDetails: _List[AllowanceDetails] = []


class TaxDetails(__BaseDictObject):
    """
    Details of tax amount applied.
    """

    def __init__(self, data):
        super().__init__(data)
        if "taxType" in data:
            self.taxType: str = self._get_value(str, "taxType")
        else:
            self.taxType: str = None
        if "taxRate" in data:
            self.taxRate: Decimal = self._get_value(Decimal, "taxRate")
        else:
            self.taxRate: Decimal = None
        if "taxAmount" in data:
            self.taxAmount: Money = self._get_value(Money, "taxAmount")
        else:
            self.taxAmount: Money = None
        if "taxableAmount" in data:
            self.taxableAmount: Money = self._get_value(Money, "taxableAmount")
        else:
            self.taxableAmount: Money = None


class Money(__BaseDictObject):
    """
    An amount of money, including units in the form of currency.
    """

    def __init__(self, data):
        super().__init__(data)
        if "currencyCode" in data:
            self.currencyCode: str = self._get_value(str, "currencyCode")
        else:
            self.currencyCode: str = None
        if "amount" in data:
            self.amount: Decimal = self._get_value(Decimal, "amount")
        else:
            self.amount: Decimal = None


class AdditionalDetails(__BaseDictObject):
    """
    Additional information provided by the selling party for tax-related or any other purpose.
    """

    def __init__(self, data):
        super().__init__(data)
        if "type" in data:
            self.type: str = self._get_value(str, "type")
        else:
            self.type: str = None
        if "detail" in data:
            self.detail: str = self._get_value(str, "detail")
        else:
            self.detail: str = None
        if "languageCode" in data:
            self.languageCode: str = self._get_value(str, "languageCode")
        else:
            self.languageCode: str = None


class ChargeDetails(__BaseDictObject):
    """
    Monetary and tax details of the charge.
    """

    def __init__(self, data):
        super().__init__(data)
        if "type" in data:
            self.type: str = self._get_value(str, "type")
        else:
            self.type: str = None
        if "description" in data:
            self.description: str = self._get_value(str, "description")
        else:
            self.description: str = None
        if "chargeAmount" in data:
            self.chargeAmount: Money = self._get_value(Money, "chargeAmount")
        else:
            self.chargeAmount: Money = None
        if "taxDetails" in data:
            self.taxDetails: _List[TaxDetails] = [TaxDetails(datum) for datum in data["taxDetails"]]
        else:
            self.taxDetails: _List[TaxDetails] = []


class AllowanceDetails(__BaseDictObject):
    """
    Monetary and tax details of the allowance.
    """

    def __init__(self, data):
        super().__init__(data)
        if "type" in data:
            self.type: str = self._get_value(str, "type")
        else:
            self.type: str = None
        if "description" in data:
            self.description: str = self._get_value(str, "description")
        else:
            self.description: str = None
        if "allowanceAmount" in data:
            self.allowanceAmount: Money = self._get_value(Money, "allowanceAmount")
        else:
            self.allowanceAmount: Money = None
        if "taxDetails" in data:
            self.taxDetails: _List[TaxDetails] = [TaxDetails(datum) for datum in data["taxDetails"]]
        else:
            self.taxDetails: _List[TaxDetails] = []


class PaymentTerms(__BaseDictObject):
    """
    Terms of the payment for the invoice. The basis of the payment terms is the invoice date.
    """

    def __init__(self, data):
        super().__init__(data)
        if "type" in data:
            self.type: str = self._get_value(str, "type")
        else:
            self.type: str = None
        if "discountPercent" in data:
            self.discountPercent: Decimal = self._get_value(Decimal, "discountPercent")
        else:
            self.discountPercent: Decimal = None
        if "discountDueDays" in data:
            self.discountDueDays: float = self._get_value(float, "discountDueDays")
        else:
            self.discountDueDays: float = None
        if "netDueDays" in data:
            self.netDueDays: float = self._get_value(float, "netDueDays")
        else:
            self.netDueDays: float = None


class CreditNoteDetails(__BaseDictObject):
    """
    References required in order to process a credit note. This information is required only if InvoiceType is CreditNote.
    """

    def __init__(self, data):
        super().__init__(data)
        if "referenceInvoiceNumber" in data:
            self.referenceInvoiceNumber: str = self._get_value(str, "referenceInvoiceNumber")
        else:
            self.referenceInvoiceNumber: str = None
        if "debitNoteNumber" in data:
            self.debitNoteNumber: str = self._get_value(str, "debitNoteNumber")
        else:
            self.debitNoteNumber: str = None
        if "returnsReferenceNumber" in data:
            self.returnsReferenceNumber: str = self._get_value(str, "returnsReferenceNumber")
        else:
            self.returnsReferenceNumber: str = None
        if "goodsReturnDate" in data:
            self.goodsReturnDate: DateTime = self._get_value(DateTime, "goodsReturnDate")
        else:
            self.goodsReturnDate: DateTime = None
        if "rmaId" in data:
            self.rmaId: str = self._get_value(str, "rmaId")
        else:
            self.rmaId: str = None
        if "coopReferenceNumber" in data:
            self.coopReferenceNumber: str = self._get_value(str, "coopReferenceNumber")
        else:
            self.coopReferenceNumber: str = None
        if "consignorsReferenceNumber" in data:
            self.consignorsReferenceNumber: str = self._get_value(str, "consignorsReferenceNumber")
        else:
            self.consignorsReferenceNumber: str = None


class ItemQuantity(__BaseDictObject):
    """
    Details of quantity.
    """

    def __init__(self, data):
        super().__init__(data)
        if "amount" in data:
            self.amount: int = self._get_value(int, "amount")
        else:
            self.amount: int = None
        if "unitOfMeasure" in data:
            self.unitOfMeasure: str = self._get_value(str, "unitOfMeasure")
        else:
            self.unitOfMeasure: str = None
        if "unitSize" in data:
            self.unitSize: int = self._get_value(int, "unitSize")
        else:
            self.unitSize: int = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class Decimal(str):
    """
    A decimal number with no loss of precision. Useful when precision loss is unacceptable, as with currencies. Follows RFC7159 for number representation. <br>**Pattern** : `^-?(0|([1-9]\d*))(\.\d+)?([eE][+-]?\d+)?$`.
    """


class DateTime(str):
    """
    Defines a date and time according to ISO8601.
    """


class VendorInvoicesV1Client(__BaseClient):
    def submitInvoices(
        self,
        data: SubmitInvoicesRequest,
    ):
        """
                Submit new invoices to Amazon.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 10 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/vendor/payments/v1/invoices"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            202: SubmitInvoicesResponse,
            400: SubmitInvoicesResponse,
            403: SubmitInvoicesResponse,
            404: SubmitInvoicesResponse,
            413: SubmitInvoicesResponse,
            415: SubmitInvoicesResponse,
            429: SubmitInvoicesResponse,
            500: SubmitInvoicesResponse,
            503: SubmitInvoicesResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class UpdateShipmentStatusRequest(__BaseDictObject):
    """
    Request to update the status of shipment of an order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "marketplaceId" in data:
            self.marketplaceId: MarketplaceId = self._get_value(MarketplaceId, "marketplaceId")
        else:
            self.marketplaceId: MarketplaceId = None
        if "shipmentStatus" in data:
            self.shipmentStatus: ShipmentStatus = self._get_value(ShipmentStatus, "shipmentStatus")
        else:
            self.shipmentStatus: ShipmentStatus = None
        if "orderItems" in data:
            self.orderItems: OrderItems = self._get_value(OrderItems, "orderItems")
        else:
            self.orderItems: OrderItems = None


class UpdateVerificationStatusRequest(__BaseDictObject):
    """
    Request to update the verification status of an order containing regulated products.
    """

    def __init__(self, data):
        super().__init__(data)
        if "regulatedOrderVerificationStatus" in data:
            self.regulatedOrderVerificationStatus: UpdateVerificationStatusRequestBody = self._get_value(
                UpdateVerificationStatusRequestBody, "regulatedOrderVerificationStatus"
            )
        else:
            self.regulatedOrderVerificationStatus: UpdateVerificationStatusRequestBody = None


class UpdateVerificationStatusRequestBody(__BaseDictObject):
    """
    The updated values of the VerificationStatus field.
    """

    def __init__(self, data):
        super().__init__(data)
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "externalReviewerId" in data:
            self.externalReviewerId: str = self._get_value(str, "externalReviewerId")
        else:
            self.externalReviewerId: str = None
        if "rejectionReasonId" in data:
            self.rejectionReasonId: str = self._get_value(str, "rejectionReasonId")
        else:
            self.rejectionReasonId: str = None


class UpdateShipmentStatusErrorResponse(__BaseDictObject):
    """
    The error response schema for the UpdateShipmentStatus operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class UpdateVerificationStatusErrorResponse(__BaseDictObject):
    """
    The error response schema for the UpdateVerificationStatus operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrdersResponse(__BaseDictObject):
    """
    The response schema for the getOrders operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrdersList = self._get_value(OrdersList, "payload")
        else:
            self.payload: OrdersList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderResponse(__BaseDictObject):
    """
    The response schema for the getOrder operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: Order = self._get_value(Order, "payload")
        else:
            self.payload: Order = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderBuyerInfoResponse(__BaseDictObject):
    """
    The response schema for the getOrderBuyerInfo operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrderBuyerInfo = self._get_value(OrderBuyerInfo, "payload")
        else:
            self.payload: OrderBuyerInfo = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderRegulatedInfoResponse(__BaseDictObject):
    """
    The response schema for the getOrderRegulatedInfo operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrderRegulatedInfo = self._get_value(OrderRegulatedInfo, "payload")
        else:
            self.payload: OrderRegulatedInfo = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderAddressResponse(__BaseDictObject):
    """
    The response schema for the getOrderAddress operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrderAddress = self._get_value(OrderAddress, "payload")
        else:
            self.payload: OrderAddress = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderItemsResponse(__BaseDictObject):
    """
    The response schema for the getOrderItems operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrderItemsList = self._get_value(OrderItemsList, "payload")
        else:
            self.payload: OrderItemsList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class GetOrderItemsBuyerInfoResponse(__BaseDictObject):
    """
    The response schema for the getOrderItemsBuyerInfo operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: OrderItemsBuyerInfoList = self._get_value(OrderItemsBuyerInfoList, "payload")
        else:
            self.payload: OrderItemsBuyerInfoList = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class OrdersList(__BaseDictObject):
    """
    A list of orders along with additional information to make subsequent API calls.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Orders" in data:
            self.Orders: OrderList = self._get_value(OrderList, "Orders")
        else:
            self.Orders: OrderList = None
        if "NextToken" in data:
            self.NextToken: str = self._get_value(str, "NextToken")
        else:
            self.NextToken: str = None
        if "LastUpdatedBefore" in data:
            self.LastUpdatedBefore: str = self._get_value(str, "LastUpdatedBefore")
        else:
            self.LastUpdatedBefore: str = None
        if "CreatedBefore" in data:
            self.CreatedBefore: str = self._get_value(str, "CreatedBefore")
        else:
            self.CreatedBefore: str = None


class Order(__BaseDictObject):
    """
    Order information.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "SellerOrderId" in data:
            self.SellerOrderId: str = self._get_value(str, "SellerOrderId")
        else:
            self.SellerOrderId: str = None
        if "PurchaseDate" in data:
            self.PurchaseDate: str = self._get_value(str, "PurchaseDate")
        else:
            self.PurchaseDate: str = None
        if "LastUpdateDate" in data:
            self.LastUpdateDate: str = self._get_value(str, "LastUpdateDate")
        else:
            self.LastUpdateDate: str = None
        if "OrderStatus" in data:
            self.OrderStatus: str = self._get_value(str, "OrderStatus")
        else:
            self.OrderStatus: str = None
        if "FulfillmentChannel" in data:
            self.FulfillmentChannel: str = self._get_value(str, "FulfillmentChannel")
        else:
            self.FulfillmentChannel: str = None
        if "SalesChannel" in data:
            self.SalesChannel: str = self._get_value(str, "SalesChannel")
        else:
            self.SalesChannel: str = None
        if "OrderChannel" in data:
            self.OrderChannel: str = self._get_value(str, "OrderChannel")
        else:
            self.OrderChannel: str = None
        if "ShipServiceLevel" in data:
            self.ShipServiceLevel: str = self._get_value(str, "ShipServiceLevel")
        else:
            self.ShipServiceLevel: str = None
        if "OrderTotal" in data:
            self.OrderTotal: Money = self._get_value(Money, "OrderTotal")
        else:
            self.OrderTotal: Money = None
        if "NumberOfItemsShipped" in data:
            self.NumberOfItemsShipped: int = self._get_value(int, "NumberOfItemsShipped")
        else:
            self.NumberOfItemsShipped: int = None
        if "NumberOfItemsUnshipped" in data:
            self.NumberOfItemsUnshipped: int = self._get_value(int, "NumberOfItemsUnshipped")
        else:
            self.NumberOfItemsUnshipped: int = None
        if "PaymentExecutionDetail" in data:
            self.PaymentExecutionDetail: PaymentExecutionDetailItemList = self._get_value(
                PaymentExecutionDetailItemList, "PaymentExecutionDetail"
            )
        else:
            self.PaymentExecutionDetail: PaymentExecutionDetailItemList = None
        if "PaymentMethod" in data:
            self.PaymentMethod: str = self._get_value(str, "PaymentMethod")
        else:
            self.PaymentMethod: str = None
        if "PaymentMethodDetails" in data:
            self.PaymentMethodDetails: PaymentMethodDetailItemList = self._get_value(
                PaymentMethodDetailItemList, "PaymentMethodDetails"
            )
        else:
            self.PaymentMethodDetails: PaymentMethodDetailItemList = None
        if "MarketplaceId" in data:
            self.MarketplaceId: str = self._get_value(str, "MarketplaceId")
        else:
            self.MarketplaceId: str = None
        if "ShipmentServiceLevelCategory" in data:
            self.ShipmentServiceLevelCategory: str = self._get_value(str, "ShipmentServiceLevelCategory")
        else:
            self.ShipmentServiceLevelCategory: str = None
        if "EasyShipShipmentStatus" in data:
            self.EasyShipShipmentStatus: str = self._get_value(str, "EasyShipShipmentStatus")
        else:
            self.EasyShipShipmentStatus: str = None
        if "CbaDisplayableShippingLabel" in data:
            self.CbaDisplayableShippingLabel: str = self._get_value(str, "CbaDisplayableShippingLabel")
        else:
            self.CbaDisplayableShippingLabel: str = None
        if "OrderType" in data:
            self.OrderType: str = self._get_value(str, "OrderType")
        else:
            self.OrderType: str = None
        if "EarliestShipDate" in data:
            self.EarliestShipDate: str = self._get_value(str, "EarliestShipDate")
        else:
            self.EarliestShipDate: str = None
        if "LatestShipDate" in data:
            self.LatestShipDate: str = self._get_value(str, "LatestShipDate")
        else:
            self.LatestShipDate: str = None
        if "EarliestDeliveryDate" in data:
            self.EarliestDeliveryDate: str = self._get_value(str, "EarliestDeliveryDate")
        else:
            self.EarliestDeliveryDate: str = None
        if "LatestDeliveryDate" in data:
            self.LatestDeliveryDate: str = self._get_value(str, "LatestDeliveryDate")
        else:
            self.LatestDeliveryDate: str = None
        if "IsBusinessOrder" in data:
            self.IsBusinessOrder: bool = self._get_value(convert_bool, "IsBusinessOrder")
        else:
            self.IsBusinessOrder: bool = None
        if "IsPrime" in data:
            self.IsPrime: bool = self._get_value(convert_bool, "IsPrime")
        else:
            self.IsPrime: bool = None
        if "IsPremiumOrder" in data:
            self.IsPremiumOrder: bool = self._get_value(convert_bool, "IsPremiumOrder")
        else:
            self.IsPremiumOrder: bool = None
        if "IsGlobalExpressEnabled" in data:
            self.IsGlobalExpressEnabled: bool = self._get_value(convert_bool, "IsGlobalExpressEnabled")
        else:
            self.IsGlobalExpressEnabled: bool = None
        if "ReplacedOrderId" in data:
            self.ReplacedOrderId: str = self._get_value(str, "ReplacedOrderId")
        else:
            self.ReplacedOrderId: str = None
        if "IsReplacementOrder" in data:
            self.IsReplacementOrder: bool = self._get_value(convert_bool, "IsReplacementOrder")
        else:
            self.IsReplacementOrder: bool = None
        if "PromiseResponseDueDate" in data:
            self.PromiseResponseDueDate: str = self._get_value(str, "PromiseResponseDueDate")
        else:
            self.PromiseResponseDueDate: str = None
        if "IsEstimatedShipDateSet" in data:
            self.IsEstimatedShipDateSet: bool = self._get_value(convert_bool, "IsEstimatedShipDateSet")
        else:
            self.IsEstimatedShipDateSet: bool = None
        if "IsSoldByAB" in data:
            self.IsSoldByAB: bool = self._get_value(convert_bool, "IsSoldByAB")
        else:
            self.IsSoldByAB: bool = None
        if "IsIBA" in data:
            self.IsIBA: bool = self._get_value(convert_bool, "IsIBA")
        else:
            self.IsIBA: bool = None
        if "DefaultShipFromLocationAddress" in data:
            self.DefaultShipFromLocationAddress: Address = self._get_value(Address, "DefaultShipFromLocationAddress")
        else:
            self.DefaultShipFromLocationAddress: Address = None
        if "BuyerInvoicePreference" in data:
            self.BuyerInvoicePreference: str = self._get_value(str, "BuyerInvoicePreference")
        else:
            self.BuyerInvoicePreference: str = None
        if "BuyerTaxInformation" in data:
            self.BuyerTaxInformation: BuyerTaxInformation = self._get_value(BuyerTaxInformation, "BuyerTaxInformation")
        else:
            self.BuyerTaxInformation: BuyerTaxInformation = None
        if "FulfillmentInstruction" in data:
            self.FulfillmentInstruction: FulfillmentInstruction = self._get_value(
                FulfillmentInstruction, "FulfillmentInstruction"
            )
        else:
            self.FulfillmentInstruction: FulfillmentInstruction = None
        if "IsISPU" in data:
            self.IsISPU: bool = self._get_value(convert_bool, "IsISPU")
        else:
            self.IsISPU: bool = None
        if "MarketplaceTaxInfo" in data:
            self.MarketplaceTaxInfo: MarketplaceTaxInfo = self._get_value(MarketplaceTaxInfo, "MarketplaceTaxInfo")
        else:
            self.MarketplaceTaxInfo: MarketplaceTaxInfo = None
        if "SellerDisplayName" in data:
            self.SellerDisplayName: str = self._get_value(str, "SellerDisplayName")
        else:
            self.SellerDisplayName: str = None
        if "ShippingAddress" in data:
            self.ShippingAddress: Address = self._get_value(Address, "ShippingAddress")
        else:
            self.ShippingAddress: Address = None
        if "BuyerInfo" in data:
            self.BuyerInfo: BuyerInfo = self._get_value(BuyerInfo, "BuyerInfo")
        else:
            self.BuyerInfo: BuyerInfo = None
        if "AutomatedShippingSettings" in data:
            self.AutomatedShippingSettings: AutomatedShippingSettings = self._get_value(
                AutomatedShippingSettings, "AutomatedShippingSettings"
            )
        else:
            self.AutomatedShippingSettings: AutomatedShippingSettings = None
        if "HasRegulatedItems" in data:
            self.HasRegulatedItems: bool = self._get_value(convert_bool, "HasRegulatedItems")
        else:
            self.HasRegulatedItems: bool = None


class OrderBuyerInfo(__BaseDictObject):
    """
    Buyer information for an order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "BuyerEmail" in data:
            self.BuyerEmail: str = self._get_value(str, "BuyerEmail")
        else:
            self.BuyerEmail: str = None
        if "BuyerName" in data:
            self.BuyerName: str = self._get_value(str, "BuyerName")
        else:
            self.BuyerName: str = None
        if "BuyerCounty" in data:
            self.BuyerCounty: str = self._get_value(str, "BuyerCounty")
        else:
            self.BuyerCounty: str = None
        if "BuyerTaxInfo" in data:
            self.BuyerTaxInfo: BuyerTaxInfo = self._get_value(BuyerTaxInfo, "BuyerTaxInfo")
        else:
            self.BuyerTaxInfo: BuyerTaxInfo = None
        if "PurchaseOrderNumber" in data:
            self.PurchaseOrderNumber: str = self._get_value(str, "PurchaseOrderNumber")
        else:
            self.PurchaseOrderNumber: str = None


class OrderRegulatedInfo(__BaseDictObject):
    """
    The order's regulated information along with its verification status.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "RegulatedInformation" in data:
            self.RegulatedInformation: RegulatedInformation = self._get_value(
                RegulatedInformation, "RegulatedInformation"
            )
        else:
            self.RegulatedInformation: RegulatedInformation = None
        if "RequiresDosageLabel" in data:
            self.RequiresDosageLabel: bool = self._get_value(convert_bool, "RequiresDosageLabel")
        else:
            self.RequiresDosageLabel: bool = None
        if "RegulatedOrderVerificationStatus" in data:
            self.RegulatedOrderVerificationStatus: RegulatedOrderVerificationStatus = self._get_value(
                RegulatedOrderVerificationStatus, "RegulatedOrderVerificationStatus"
            )
        else:
            self.RegulatedOrderVerificationStatus: RegulatedOrderVerificationStatus = None


class RegulatedOrderVerificationStatus(__BaseDictObject):
    """
    The verification status of the order along with associated approval or rejection metadata.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Status" in data:
            self.Status: str = self._get_value(str, "Status")
        else:
            self.Status: str = None
        if "RequiresMerchantAction" in data:
            self.RequiresMerchantAction: bool = self._get_value(convert_bool, "RequiresMerchantAction")
        else:
            self.RequiresMerchantAction: bool = None
        if "ValidRejectionReasons" in data:
            self.ValidRejectionReasons: _List[RejectionReason] = [
                RejectionReason(datum) for datum in data["ValidRejectionReasons"]
            ]
        else:
            self.ValidRejectionReasons: _List[RejectionReason] = []
        if "RejectionReason" in data:
            self.RejectionReason: RejectionReason = self._get_value(RejectionReason, "RejectionReason")
        else:
            self.RejectionReason: RejectionReason = None
        if "ReviewDate" in data:
            self.ReviewDate: str = self._get_value(str, "ReviewDate")
        else:
            self.ReviewDate: str = None
        if "ExternalReviewerId" in data:
            self.ExternalReviewerId: str = self._get_value(str, "ExternalReviewerId")
        else:
            self.ExternalReviewerId: str = None


class RejectionReason(__BaseDictObject):
    """
    The reason for rejecting the order's regulated information. Not present if the order isn't rejected.
    """

    def __init__(self, data):
        super().__init__(data)
        if "RejectionReasonId" in data:
            self.RejectionReasonId: str = self._get_value(str, "RejectionReasonId")
        else:
            self.RejectionReasonId: str = None
        if "RejectionReasonDescription" in data:
            self.RejectionReasonDescription: str = self._get_value(str, "RejectionReasonDescription")
        else:
            self.RejectionReasonDescription: str = None


class RegulatedInformation(__BaseDictObject):
    """
    The regulated information collected during purchase and used to verify the order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Fields" in data:
            self.Fields: _List[RegulatedInformationField] = [
                RegulatedInformationField(datum) for datum in data["Fields"]
            ]
        else:
            self.Fields: _List[RegulatedInformationField] = []


class RegulatedInformationField(__BaseDictObject):
    """
    A field collected from the regulatory form.
    """

    def __init__(self, data):
        super().__init__(data)
        if "FieldId" in data:
            self.FieldId: str = self._get_value(str, "FieldId")
        else:
            self.FieldId: str = None
        if "FieldLabel" in data:
            self.FieldLabel: str = self._get_value(str, "FieldLabel")
        else:
            self.FieldLabel: str = None
        if "FieldType" in data:
            self.FieldType: str = self._get_value(str, "FieldType")
        else:
            self.FieldType: str = None
        if "FieldValue" in data:
            self.FieldValue: str = self._get_value(str, "FieldValue")
        else:
            self.FieldValue: str = None


class OrderAddress(__BaseDictObject):
    """
    The shipping address for the order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None
        if "ShippingAddress" in data:
            self.ShippingAddress: Address = self._get_value(Address, "ShippingAddress")
        else:
            self.ShippingAddress: Address = None


class Address(__BaseDictObject):
    """
    The shipping address for the order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Name" in data:
            self.Name: str = self._get_value(str, "Name")
        else:
            self.Name: str = None
        if "AddressLine1" in data:
            self.AddressLine1: str = self._get_value(str, "AddressLine1")
        else:
            self.AddressLine1: str = None
        if "AddressLine2" in data:
            self.AddressLine2: str = self._get_value(str, "AddressLine2")
        else:
            self.AddressLine2: str = None
        if "AddressLine3" in data:
            self.AddressLine3: str = self._get_value(str, "AddressLine3")
        else:
            self.AddressLine3: str = None
        if "City" in data:
            self.City: str = self._get_value(str, "City")
        else:
            self.City: str = None
        if "County" in data:
            self.County: str = self._get_value(str, "County")
        else:
            self.County: str = None
        if "District" in data:
            self.District: str = self._get_value(str, "District")
        else:
            self.District: str = None
        if "StateOrRegion" in data:
            self.StateOrRegion: str = self._get_value(str, "StateOrRegion")
        else:
            self.StateOrRegion: str = None
        if "Municipality" in data:
            self.Municipality: str = self._get_value(str, "Municipality")
        else:
            self.Municipality: str = None
        if "PostalCode" in data:
            self.PostalCode: str = self._get_value(str, "PostalCode")
        else:
            self.PostalCode: str = None
        if "CountryCode" in data:
            self.CountryCode: str = self._get_value(str, "CountryCode")
        else:
            self.CountryCode: str = None
        if "Phone" in data:
            self.Phone: str = self._get_value(str, "Phone")
        else:
            self.Phone: str = None
        if "AddressType" in data:
            self.AddressType: str = self._get_value(str, "AddressType")
        else:
            self.AddressType: str = None


class Money(__BaseDictObject):
    """
    The monetary value of the order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "CurrencyCode" in data:
            self.CurrencyCode: str = self._get_value(str, "CurrencyCode")
        else:
            self.CurrencyCode: str = None
        if "Amount" in data:
            self.Amount: str = self._get_value(str, "Amount")
        else:
            self.Amount: str = None


class PaymentExecutionDetailItem(__BaseDictObject):
    """
    Information about a sub-payment method used to pay for a COD order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Payment" in data:
            self.Payment: Money = self._get_value(Money, "Payment")
        else:
            self.Payment: Money = None
        if "PaymentMethod" in data:
            self.PaymentMethod: str = self._get_value(str, "PaymentMethod")
        else:
            self.PaymentMethod: str = None


class BuyerTaxInfo(__BaseDictObject):
    """
    Tax information about the buyer.
    """

    def __init__(self, data):
        super().__init__(data)
        if "CompanyLegalName" in data:
            self.CompanyLegalName: str = self._get_value(str, "CompanyLegalName")
        else:
            self.CompanyLegalName: str = None
        if "TaxingRegion" in data:
            self.TaxingRegion: str = self._get_value(str, "TaxingRegion")
        else:
            self.TaxingRegion: str = None
        if "TaxClassifications" in data:
            self.TaxClassifications: _List[TaxClassification] = [
                TaxClassification(datum) for datum in data["TaxClassifications"]
            ]
        else:
            self.TaxClassifications: _List[TaxClassification] = []


class MarketplaceTaxInfo(__BaseDictObject):
    """
    Tax information about the marketplace.
    """

    def __init__(self, data):
        super().__init__(data)
        if "TaxClassifications" in data:
            self.TaxClassifications: _List[TaxClassification] = [
                TaxClassification(datum) for datum in data["TaxClassifications"]
            ]
        else:
            self.TaxClassifications: _List[TaxClassification] = []


class TaxClassification(__BaseDictObject):
    """
    The tax classification for the order.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Name" in data:
            self.Name: str = self._get_value(str, "Name")
        else:
            self.Name: str = None
        if "Value" in data:
            self.Value: str = self._get_value(str, "Value")
        else:
            self.Value: str = None


class OrderItemsList(__BaseDictObject):
    """
    The order items list along with the order ID.
    """

    def __init__(self, data):
        super().__init__(data)
        if "OrderItems" in data:
            self.OrderItems: OrderItemList = self._get_value(OrderItemList, "OrderItems")
        else:
            self.OrderItems: OrderItemList = None
        if "NextToken" in data:
            self.NextToken: str = self._get_value(str, "NextToken")
        else:
            self.NextToken: str = None
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None


class OrderItem(__BaseDictObject):
    """
    A single order item.
    """

    def __init__(self, data):
        super().__init__(data)
        if "ASIN" in data:
            self.ASIN: str = self._get_value(str, "ASIN")
        else:
            self.ASIN: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = self._get_value(str, "SellerSKU")
        else:
            self.SellerSKU: str = None
        if "OrderItemId" in data:
            self.OrderItemId: str = self._get_value(str, "OrderItemId")
        else:
            self.OrderItemId: str = None
        if "Title" in data:
            self.Title: str = self._get_value(str, "Title")
        else:
            self.Title: str = None
        if "QuantityOrdered" in data:
            self.QuantityOrdered: int = self._get_value(int, "QuantityOrdered")
        else:
            self.QuantityOrdered: int = None
        if "QuantityShipped" in data:
            self.QuantityShipped: int = self._get_value(int, "QuantityShipped")
        else:
            self.QuantityShipped: int = None
        if "ProductInfo" in data:
            self.ProductInfo: ProductInfoDetail = self._get_value(ProductInfoDetail, "ProductInfo")
        else:
            self.ProductInfo: ProductInfoDetail = None
        if "PointsGranted" in data:
            self.PointsGranted: PointsGrantedDetail = self._get_value(PointsGrantedDetail, "PointsGranted")
        else:
            self.PointsGranted: PointsGrantedDetail = None
        if "ItemPrice" in data:
            self.ItemPrice: Money = self._get_value(Money, "ItemPrice")
        else:
            self.ItemPrice: Money = None
        if "ShippingPrice" in data:
            self.ShippingPrice: Money = self._get_value(Money, "ShippingPrice")
        else:
            self.ShippingPrice: Money = None
        if "ItemTax" in data:
            self.ItemTax: Money = self._get_value(Money, "ItemTax")
        else:
            self.ItemTax: Money = None
        if "ShippingTax" in data:
            self.ShippingTax: Money = self._get_value(Money, "ShippingTax")
        else:
            self.ShippingTax: Money = None
        if "ShippingDiscount" in data:
            self.ShippingDiscount: Money = self._get_value(Money, "ShippingDiscount")
        else:
            self.ShippingDiscount: Money = None
        if "ShippingDiscountTax" in data:
            self.ShippingDiscountTax: Money = self._get_value(Money, "ShippingDiscountTax")
        else:
            self.ShippingDiscountTax: Money = None
        if "PromotionDiscount" in data:
            self.PromotionDiscount: Money = self._get_value(Money, "PromotionDiscount")
        else:
            self.PromotionDiscount: Money = None
        if "PromotionDiscountTax" in data:
            self.PromotionDiscountTax: Money = self._get_value(Money, "PromotionDiscountTax")
        else:
            self.PromotionDiscountTax: Money = None
        if "PromotionIds" in data:
            self.PromotionIds: PromotionIdList = self._get_value(PromotionIdList, "PromotionIds")
        else:
            self.PromotionIds: PromotionIdList = None
        if "CODFee" in data:
            self.CODFee: Money = self._get_value(Money, "CODFee")
        else:
            self.CODFee: Money = None
        if "CODFeeDiscount" in data:
            self.CODFeeDiscount: Money = self._get_value(Money, "CODFeeDiscount")
        else:
            self.CODFeeDiscount: Money = None
        if "IsGift" in data:
            self.IsGift: bool = self._get_value(convert_bool, "IsGift")
        else:
            self.IsGift: bool = None
        if "ConditionNote" in data:
            self.ConditionNote: str = self._get_value(str, "ConditionNote")
        else:
            self.ConditionNote: str = None
        if "ConditionId" in data:
            self.ConditionId: str = self._get_value(str, "ConditionId")
        else:
            self.ConditionId: str = None
        if "ConditionSubtypeId" in data:
            self.ConditionSubtypeId: str = self._get_value(str, "ConditionSubtypeId")
        else:
            self.ConditionSubtypeId: str = None
        if "ScheduledDeliveryStartDate" in data:
            self.ScheduledDeliveryStartDate: str = self._get_value(str, "ScheduledDeliveryStartDate")
        else:
            self.ScheduledDeliveryStartDate: str = None
        if "ScheduledDeliveryEndDate" in data:
            self.ScheduledDeliveryEndDate: str = self._get_value(str, "ScheduledDeliveryEndDate")
        else:
            self.ScheduledDeliveryEndDate: str = None
        if "PriceDesignation" in data:
            self.PriceDesignation: str = self._get_value(str, "PriceDesignation")
        else:
            self.PriceDesignation: str = None
        if "TaxCollection" in data:
            self.TaxCollection: TaxCollection = self._get_value(TaxCollection, "TaxCollection")
        else:
            self.TaxCollection: TaxCollection = None
        if "SerialNumberRequired" in data:
            self.SerialNumberRequired: bool = self._get_value(convert_bool, "SerialNumberRequired")
        else:
            self.SerialNumberRequired: bool = None
        if "IsTransparency" in data:
            self.IsTransparency: bool = self._get_value(convert_bool, "IsTransparency")
        else:
            self.IsTransparency: bool = None
        if "IossNumber" in data:
            self.IossNumber: str = self._get_value(str, "IossNumber")
        else:
            self.IossNumber: str = None
        if "StoreChainStoreId" in data:
            self.StoreChainStoreId: str = self._get_value(str, "StoreChainStoreId")
        else:
            self.StoreChainStoreId: str = None
        if "DeemedResellerCategory" in data:
            self.DeemedResellerCategory: str = self._get_value(str, "DeemedResellerCategory")
        else:
            self.DeemedResellerCategory: str = None
        if "BuyerInfo" in data:
            self.BuyerInfo: ItemBuyerInfo = self._get_value(ItemBuyerInfo, "BuyerInfo")
        else:
            self.BuyerInfo: ItemBuyerInfo = None
        if "BuyerRequestedCancel" in data:
            self.BuyerRequestedCancel: BuyerRequestedCancel = self._get_value(
                BuyerRequestedCancel, "BuyerRequestedCancel"
            )
        else:
            self.BuyerRequestedCancel: BuyerRequestedCancel = None


class OrderItemsBuyerInfoList(__BaseDictObject):
    """
    A single order item's buyer information list with the order ID.
    """

    def __init__(self, data):
        super().__init__(data)
        if "OrderItems" in data:
            self.OrderItems: OrderItemBuyerInfoList = self._get_value(OrderItemBuyerInfoList, "OrderItems")
        else:
            self.OrderItems: OrderItemBuyerInfoList = None
        if "NextToken" in data:
            self.NextToken: str = self._get_value(str, "NextToken")
        else:
            self.NextToken: str = None
        if "AmazonOrderId" in data:
            self.AmazonOrderId: str = self._get_value(str, "AmazonOrderId")
        else:
            self.AmazonOrderId: str = None


class OrderItemBuyerInfo(__BaseDictObject):
    """
    A single order item's buyer information.
    """

    def __init__(self, data):
        super().__init__(data)
        if "OrderItemId" in data:
            self.OrderItemId: str = self._get_value(str, "OrderItemId")
        else:
            self.OrderItemId: str = None
        if "BuyerCustomizedInfo" in data:
            self.BuyerCustomizedInfo: BuyerCustomizedInfoDetail = self._get_value(
                BuyerCustomizedInfoDetail, "BuyerCustomizedInfo"
            )
        else:
            self.BuyerCustomizedInfo: BuyerCustomizedInfoDetail = None
        if "GiftWrapPrice" in data:
            self.GiftWrapPrice: Money = self._get_value(Money, "GiftWrapPrice")
        else:
            self.GiftWrapPrice: Money = None
        if "GiftWrapTax" in data:
            self.GiftWrapTax: Money = self._get_value(Money, "GiftWrapTax")
        else:
            self.GiftWrapTax: Money = None
        if "GiftMessageText" in data:
            self.GiftMessageText: str = self._get_value(str, "GiftMessageText")
        else:
            self.GiftMessageText: str = None
        if "GiftWrapLevel" in data:
            self.GiftWrapLevel: str = self._get_value(str, "GiftWrapLevel")
        else:
            self.GiftWrapLevel: str = None


class PointsGrantedDetail(__BaseDictObject):
    """
    The number of Amazon Points offered with the purchase of an item, and their monetary value.
    """

    def __init__(self, data):
        super().__init__(data)
        if "PointsNumber" in data:
            self.PointsNumber: int = self._get_value(int, "PointsNumber")
        else:
            self.PointsNumber: int = None
        if "PointsMonetaryValue" in data:
            self.PointsMonetaryValue: Money = self._get_value(Money, "PointsMonetaryValue")
        else:
            self.PointsMonetaryValue: Money = None


class ProductInfoDetail(__BaseDictObject):
    """
    Product information on the number of items.
    """

    def __init__(self, data):
        super().__init__(data)
        if "NumberOfItems" in data:
            self.NumberOfItems: int = self._get_value(int, "NumberOfItems")
        else:
            self.NumberOfItems: int = None


class BuyerCustomizedInfoDetail(__BaseDictObject):
    """
    Buyer information for custom orders from the Amazon Custom program.
    """

    def __init__(self, data):
        super().__init__(data)
        if "CustomizedURL" in data:
            self.CustomizedURL: str = self._get_value(str, "CustomizedURL")
        else:
            self.CustomizedURL: str = None


class TaxCollection(__BaseDictObject):
    """
    Information about withheld taxes.
    """

    def __init__(self, data):
        super().__init__(data)
        if "Model" in data:
            self.Model: str = self._get_value(str, "Model")
        else:
            self.Model: str = None
        if "ResponsibleParty" in data:
            self.ResponsibleParty: str = self._get_value(str, "ResponsibleParty")
        else:
            self.ResponsibleParty: str = None


class BuyerTaxInformation(__BaseDictObject):
    """
    Contains the business invoice tax information. Available only in the TR marketplace.
    """

    def __init__(self, data):
        super().__init__(data)
        if "BuyerLegalCompanyName" in data:
            self.BuyerLegalCompanyName: str = self._get_value(str, "BuyerLegalCompanyName")
        else:
            self.BuyerLegalCompanyName: str = None
        if "BuyerBusinessAddress" in data:
            self.BuyerBusinessAddress: str = self._get_value(str, "BuyerBusinessAddress")
        else:
            self.BuyerBusinessAddress: str = None
        if "BuyerTaxRegistrationId" in data:
            self.BuyerTaxRegistrationId: str = self._get_value(str, "BuyerTaxRegistrationId")
        else:
            self.BuyerTaxRegistrationId: str = None
        if "BuyerTaxOffice" in data:
            self.BuyerTaxOffice: str = self._get_value(str, "BuyerTaxOffice")
        else:
            self.BuyerTaxOffice: str = None


class FulfillmentInstruction(__BaseDictObject):
    """
    Contains the instructions about the fulfillment like where should it be fulfilled from.
    """

    def __init__(self, data):
        super().__init__(data)
        if "FulfillmentSupplySourceId" in data:
            self.FulfillmentSupplySourceId: str = self._get_value(str, "FulfillmentSupplySourceId")
        else:
            self.FulfillmentSupplySourceId: str = None


class BuyerInfo(__BaseDictObject):
    """
    Buyer information
    """

    def __init__(self, data):
        super().__init__(data)
        if "BuyerEmail" in data:
            self.BuyerEmail: str = self._get_value(str, "BuyerEmail")
        else:
            self.BuyerEmail: str = None
        if "BuyerName" in data:
            self.BuyerName: str = self._get_value(str, "BuyerName")
        else:
            self.BuyerName: str = None
        if "BuyerCounty" in data:
            self.BuyerCounty: str = self._get_value(str, "BuyerCounty")
        else:
            self.BuyerCounty: str = None
        if "BuyerTaxInfo" in data:
            self.BuyerTaxInfo: BuyerTaxInfo = self._get_value(BuyerTaxInfo, "BuyerTaxInfo")
        else:
            self.BuyerTaxInfo: BuyerTaxInfo = None
        if "PurchaseOrderNumber" in data:
            self.PurchaseOrderNumber: str = self._get_value(str, "PurchaseOrderNumber")
        else:
            self.PurchaseOrderNumber: str = None


class ItemBuyerInfo(__BaseDictObject):
    """
    A single item's buyer information.
    """

    def __init__(self, data):
        super().__init__(data)
        if "BuyerCustomizedInfo" in data:
            self.BuyerCustomizedInfo: BuyerCustomizedInfoDetail = self._get_value(
                BuyerCustomizedInfoDetail, "BuyerCustomizedInfo"
            )
        else:
            self.BuyerCustomizedInfo: BuyerCustomizedInfoDetail = None
        if "GiftWrapPrice" in data:
            self.GiftWrapPrice: Money = self._get_value(Money, "GiftWrapPrice")
        else:
            self.GiftWrapPrice: Money = None
        if "GiftWrapTax" in data:
            self.GiftWrapTax: Money = self._get_value(Money, "GiftWrapTax")
        else:
            self.GiftWrapTax: Money = None
        if "GiftMessageText" in data:
            self.GiftMessageText: str = self._get_value(str, "GiftMessageText")
        else:
            self.GiftMessageText: str = None
        if "GiftWrapLevel" in data:
            self.GiftWrapLevel: str = self._get_value(str, "GiftWrapLevel")
        else:
            self.GiftWrapLevel: str = None


class AutomatedShippingSettings(__BaseDictObject):
    """
    Contains information regarding the Shipping Settings Automation program, such as whether the order's shipping settings were generated automatically, and what those settings are.
    """

    def __init__(self, data):
        super().__init__(data)
        if "HasAutomatedShippingSettings" in data:
            self.HasAutomatedShippingSettings: bool = self._get_value(convert_bool, "HasAutomatedShippingSettings")
        else:
            self.HasAutomatedShippingSettings: bool = None
        if "AutomatedCarrier" in data:
            self.AutomatedCarrier: str = self._get_value(str, "AutomatedCarrier")
        else:
            self.AutomatedCarrier: str = None
        if "AutomatedShipMethod" in data:
            self.AutomatedShipMethod: str = self._get_value(str, "AutomatedShipMethod")
        else:
            self.AutomatedShipMethod: str = None


class BuyerRequestedCancel(__BaseDictObject):
    """
    Information about whether or not a buyer requested cancellation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "IsBuyerRequestedCancel" in data:
            self.IsBuyerRequestedCancel: bool = self._get_value(convert_bool, "IsBuyerRequestedCancel")
        else:
            self.IsBuyerRequestedCancel: bool = None
        if "BuyerCancelReason" in data:
            self.BuyerCancelReason: str = self._get_value(str, "BuyerCancelReason")
        else:
            self.BuyerCancelReason: str = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class OrderItems(list, _List["dict"]):
    """
    the list of order items and quantities when the seller wants to partially update the shipment status of the order
    """

    def __init__(self, data):
        super().__init__([dict(datum) for datum in data])
        self.data = data


class OrderList(list, _List["Order"]):
    """
    A list of orders.
    """

    def __init__(self, data):
        super().__init__([Order(datum) for datum in data])
        self.data = data


class PaymentMethodDetailItemList(list, _List["str"]):
    """
    A list of payment method detail items.
    """

    def __init__(self, data):
        super().__init__([str(datum) for datum in data])
        self.data = data


class PaymentExecutionDetailItemList(list, _List["PaymentExecutionDetailItem"]):
    """
    A list of payment execution detail items.
    """

    def __init__(self, data):
        super().__init__([PaymentExecutionDetailItem(datum) for datum in data])
        self.data = data


class OrderItemList(list, _List["OrderItem"]):
    """
    A list of order items.
    """

    def __init__(self, data):
        super().__init__([OrderItem(datum) for datum in data])
        self.data = data


class OrderItemBuyerInfoList(list, _List["OrderItemBuyerInfo"]):
    """
    A single order item's buyer information list.
    """

    def __init__(self, data):
        super().__init__([OrderItemBuyerInfo(datum) for datum in data])
        self.data = data


class PromotionIdList(list, _List["str"]):
    """
    A list of promotion identifiers provided by the seller when the promotions were created.
    """

    def __init__(self, data):
        super().__init__([str(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class MarketplaceId(str):
    """
    the unobfuscated marketplace ID
    """


class ShipmentStatus(str):
    """
    the status of the shipment of the order to be updated
    """


class OrdersV0Client(__BaseClient):
    def getOrders(
        self,
        MarketplaceIds: _List[str],
        CreatedAfter: str = None,
        CreatedBefore: str = None,
        LastUpdatedAfter: str = None,
        LastUpdatedBefore: str = None,
        OrderStatuses: _List[str] = None,
        FulfillmentChannels: _List[str] = None,
        PaymentMethods: _List[str] = None,
        BuyerEmail: str = None,
        SellerOrderId: str = None,
        MaxResultsPerPage: int = None,
        EasyShipShipmentStatuses: _List[str] = None,
        NextToken: str = None,
        AmazonOrderIds: _List[str] = None,
        ActualFulfillmentSupplySourceId: str = None,
        IsISPU: bool = None,
        StoreChainStoreId: str = None,
    ):
        """
                Returns orders created or updated during the time frame indicated by the specified parameters. You can also apply a range of filtering criteria to narrow the list of orders returned. If NextToken is present, that will be used to retrieve the orders instead of other criteria.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders"
        params = {}
        if CreatedAfter is not None:
            params["CreatedAfter"] = CreatedAfter
        if CreatedBefore is not None:
            params["CreatedBefore"] = CreatedBefore
        if LastUpdatedAfter is not None:
            params["LastUpdatedAfter"] = LastUpdatedAfter
        if LastUpdatedBefore is not None:
            params["LastUpdatedBefore"] = LastUpdatedBefore
        if OrderStatuses is not None:
            params["OrderStatuses"] = ",".join(map(str, OrderStatuses))
        if MarketplaceIds is not None:
            params["MarketplaceIds"] = ",".join(map(str, MarketplaceIds))
        if FulfillmentChannels is not None:
            params["FulfillmentChannels"] = ",".join(map(str, FulfillmentChannels))
        if PaymentMethods is not None:
            params["PaymentMethods"] = ",".join(map(str, PaymentMethods))
        if BuyerEmail is not None:
            params["BuyerEmail"] = BuyerEmail
        if SellerOrderId is not None:
            params["SellerOrderId"] = SellerOrderId
        if MaxResultsPerPage is not None:
            params["MaxResultsPerPage"] = MaxResultsPerPage
        if EasyShipShipmentStatuses is not None:
            params["EasyShipShipmentStatuses"] = ",".join(map(str, EasyShipShipmentStatuses))
        if NextToken is not None:
            params["NextToken"] = NextToken
        if AmazonOrderIds is not None:
            params["AmazonOrderIds"] = ",".join(map(str, AmazonOrderIds))
        if ActualFulfillmentSupplySourceId is not None:
            params["ActualFulfillmentSupplySourceId"] = ActualFulfillmentSupplySourceId
        if IsISPU is not None:
            params["IsISPU"] = IsISPU
        if StoreChainStoreId is not None:
            params["StoreChainStoreId"] = StoreChainStoreId
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrdersResponse,
            400: GetOrdersResponse,
            403: GetOrdersResponse,
            404: GetOrdersResponse,
            429: GetOrdersResponse,
            500: GetOrdersResponse,
            503: GetOrdersResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrder(
        self,
        orderId: str,
    ):
        """
                Returns the order indicated by the specified order ID.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderResponse,
            400: GetOrderResponse,
            403: GetOrderResponse,
            404: GetOrderResponse,
            429: GetOrderResponse,
            500: GetOrderResponse,
            503: GetOrderResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrderBuyerInfo(
        self,
        orderId: str,
    ):
        """
                Returns buyer information for the specified order.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}/buyerInfo"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderBuyerInfoResponse,
            400: GetOrderBuyerInfoResponse,
            403: GetOrderBuyerInfoResponse,
            404: GetOrderBuyerInfoResponse,
            429: GetOrderBuyerInfoResponse,
            500: GetOrderBuyerInfoResponse,
            503: GetOrderBuyerInfoResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrderAddress(
        self,
        orderId: str,
    ):
        """
                Returns the shipping address for the specified order.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}/address"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderAddressResponse,
            400: GetOrderAddressResponse,
            403: GetOrderAddressResponse,
            404: GetOrderAddressResponse,
            429: GetOrderAddressResponse,
            500: GetOrderAddressResponse,
            503: GetOrderAddressResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrderItems(
        self,
        orderId: str,
        NextToken: str = None,
    ):
        """
                Returns detailed order item information for the order indicated by the specified order ID. If NextToken is provided, it's used to retrieve the next page of order items.
        Note: When an order is in the Pending state (the order has been placed but payment has not been authorized), the getOrderItems operation does not return information about pricing, taxes, shipping charges, gift status or promotions for the order items in the order. After an order leaves the Pending state (this occurs when payment has been authorized) and enters the Unshipped, Partially Shipped, or Shipped state, the getOrderItems operation returns information about pricing, taxes, shipping charges, gift status and promotions for the order items in the order.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}/orderItems"
        params = {}
        if NextToken is not None:
            params["NextToken"] = NextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderItemsResponse,
            400: GetOrderItemsResponse,
            403: GetOrderItemsResponse,
            404: GetOrderItemsResponse,
            429: GetOrderItemsResponse,
            500: GetOrderItemsResponse,
            503: GetOrderItemsResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrderItemsBuyerInfo(
        self,
        orderId: str,
        NextToken: str = None,
    ):
        """
                Returns buyer information for the order items in the specified order.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}/orderItems/buyerInfo"
        params = {}
        if NextToken is not None:
            params["NextToken"] = NextToken
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderItemsBuyerInfoResponse,
            400: GetOrderItemsBuyerInfoResponse,
            403: GetOrderItemsBuyerInfoResponse,
            404: GetOrderItemsBuyerInfoResponse,
            429: GetOrderItemsBuyerInfoResponse,
            500: GetOrderItemsBuyerInfoResponse,
            503: GetOrderItemsBuyerInfoResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def updateShipmentStatus(
        self,
        data: UpdateShipmentStatusRequest,
        orderId: str,
    ):
        """
        Update the shipment status.
        """
        url = f"/orders/v0/orders/{orderId}/shipment"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            204: None,
            400: UpdateShipmentStatusErrorResponse,
            403: UpdateShipmentStatusErrorResponse,
            404: UpdateShipmentStatusErrorResponse,
            413: UpdateShipmentStatusErrorResponse,
            415: UpdateShipmentStatusErrorResponse,
            429: UpdateShipmentStatusErrorResponse,
            500: UpdateShipmentStatusErrorResponse,
            503: UpdateShipmentStatusErrorResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrderRegulatedInfo(
        self,
        orderId: str,
    ):
        """
                Returns regulated information for the order indicated by the specified order ID.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}/regulatedInfo"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: GetOrderRegulatedInfoResponse,
            400: GetOrderRegulatedInfoResponse,
            403: GetOrderRegulatedInfoResponse,
            404: GetOrderRegulatedInfoResponse,
            429: GetOrderRegulatedInfoResponse,
            500: GetOrderRegulatedInfoResponse,
            503: GetOrderRegulatedInfoResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def updateVerificationStatus(
        self,
        data: UpdateVerificationStatusRequest,
        orderId: str,
    ):
        """
                Updates (approves or rejects) the verification status of an order containing regulated products.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 0.0055 | 20 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/orders/v0/orders/{orderId}/regulatedInfo"
        params = {}
        response = self.request(
            path=url,
            method="PATCH",
            params=params,
            data=data.data,
        )
        response_type = {
            204: None,
            400: UpdateVerificationStatusErrorResponse,
            403: UpdateVerificationStatusErrorResponse,
            404: UpdateVerificationStatusErrorResponse,
            413: UpdateVerificationStatusErrorResponse,
            415: UpdateVerificationStatusErrorResponse,
            429: UpdateVerificationStatusErrorResponse,
            500: UpdateVerificationStatusErrorResponse,
            503: UpdateVerificationStatusErrorResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class GenerateOrderScenarioRequest(__BaseDictObject):
    """
    The request body for the generateOrderScenarios operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "orders" in data:
            self.orders: _List[OrderScenarioRequest] = [OrderScenarioRequest(datum) for datum in data["orders"]]
        else:
            self.orders: _List[OrderScenarioRequest] = []


class OrderScenarioRequest(__BaseDictObject):
    """
    The party identifiers required to generate the test data.
    """

    def __init__(self, data):
        super().__init__(data)
        if "sellingParty" in data:
            self.sellingParty: PartyIdentification = self._get_value(PartyIdentification, "sellingParty")
        else:
            self.sellingParty: PartyIdentification = None
        if "shipFromParty" in data:
            self.shipFromParty: PartyIdentification = self._get_value(PartyIdentification, "shipFromParty")
        else:
            self.shipFromParty: PartyIdentification = None


class PartyIdentification(__BaseDictObject):
    """
    The identification object for the party information. For example, warehouse code or vendor code. Please refer to specific party for more details.
    """

    def __init__(self, data):
        super().__init__(data)
        if "partyId" in data:
            self.partyId: str = self._get_value(str, "partyId")
        else:
            self.partyId: str = None


class Pagination(__BaseDictObject):
    """
    A generated string used to pass information to your next request. If NextToken is returned, pass the value of NextToken to the next request. If NextToken is not returned, there are no more order items to return.
    """

    def __init__(self, data):
        super().__init__(data)
        if "nextToken" in data:
            self.nextToken: str = self._get_value(str, "nextToken")
        else:
            self.nextToken: str = None


class TransactionReference(__BaseDictObject):
    """
    A GUID assigned by Amazon to identify this transaction.
    """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None


class TransactionStatus(__BaseDictObject):
    """
    The payload for the getOrderScenarios operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "transactionStatus" in data:
            self.transactionStatus: Transaction = self._get_value(Transaction, "transactionStatus")
        else:
            self.transactionStatus: Transaction = None


class Transaction(__BaseDictObject):
    """
    The transaction details including the status. If the transaction was successful, also includes the requested test order data.
    """

    def __init__(self, data):
        super().__init__(data)
        if "transactionId" in data:
            self.transactionId: str = self._get_value(str, "transactionId")
        else:
            self.transactionId: str = None
        if "status" in data:
            self.status: str = self._get_value(str, "status")
        else:
            self.status: str = None
        if "testCaseData" in data:
            self.testCaseData: TestCaseData = self._get_value(TestCaseData, "testCaseData")
        else:
            self.testCaseData: TestCaseData = None


class TestCaseData(__BaseDictObject):
    """
    The set of test case data returned in response to the test data request.
    """

    def __init__(self, data):
        super().__init__(data)
        if "scenarios" in data:
            self.scenarios: _List[Scenario] = [Scenario(datum) for datum in data["scenarios"]]
        else:
            self.scenarios: _List[Scenario] = []


class Scenario(__BaseDictObject):
    """
    A scenario test case response returned when the request is successful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "scenarioId" in data:
            self.scenarioId: str = self._get_value(str, "scenarioId")
        else:
            self.scenarioId: str = None
        if "orders" in data:
            self.orders: _List[TestOrder] = [TestOrder(datum) for datum in data["orders"]]
        else:
            self.orders: _List[TestOrder] = []


class TestOrder(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "orderId" in data:
            self.orderId: str = self._get_value(str, "orderId")
        else:
            self.orderId: str = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class VendorDirectFulfillmentSandboxTestData20211028Client(__BaseClient):
    def generateOrderScenarios(
        self,
        data: GenerateOrderScenarioRequest,
    ):
        """
        Submits a request to generate test order data for Vendor Direct Fulfillment API entities.
        """
        url = f"/vendor/directFulfillment/sandbox/2021-10-28/orders"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            202: TransactionReference,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))

    def getOrderScenarios(
        self,
        transactionId: str,
    ):
        """
        Returns the status of the transaction indicated by the specified transactionId. If the transaction was successful, also returns the requested test order data.
        """
        url = f"/vendor/directFulfillment/sandbox/2021-10-28/transactions/{transactionId}"
        params = {}
        response = self.request(
            path=url,
            method="GET",
            params=params,
        )
        response_type = {
            200: TransactionStatus,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class CreateRestrictedDataTokenRequest(__BaseDictObject):
    """
    The request schema for the createRestrictedDataToken operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "targetApplication" in data:
            self.targetApplication: str = self._get_value(str, "targetApplication")
        else:
            self.targetApplication: str = None
        if "restrictedResources" in data:
            self.restrictedResources: _List[RestrictedResource] = [
                RestrictedResource(datum) for datum in data["restrictedResources"]
            ]
        else:
            self.restrictedResources: _List[RestrictedResource] = []


class RestrictedResource(__BaseDictObject):
    """
    Model of a restricted resource.
    """

    def __init__(self, data):
        super().__init__(data)
        if "method" in data:
            self.method: str = self._get_value(str, "method")
        else:
            self.method: str = None
        if "path" in data:
            self.path: str = self._get_value(str, "path")
        else:
            self.path: str = None
        if "dataElements" in data:
            self.dataElements: _List[str] = [str(datum) for datum in data["dataElements"]]
        else:
            self.dataElements: _List[str] = []


class CreateRestrictedDataTokenResponse(__BaseDictObject):
    """
    The response schema for the createRestrictedDataToken operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "restrictedDataToken" in data:
            self.restrictedDataToken: str = self._get_value(str, "restrictedDataToken")
        else:
            self.restrictedDataToken: str = None
        if "expiresIn" in data:
            self.expiresIn: int = self._get_value(int, "expiresIn")
        else:
            self.expiresIn: int = None


class Error(__BaseDictObject):
    """
    An error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(__BaseDictObject):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Tokens20210301Client(__BaseClient):
    def createRestrictedDataToken(
        self,
        data: CreateRestrictedDataTokenRequest,
    ):
        """
                Returns a Restricted Data Token (RDT) for one or more restricted resources that you specify. A restricted resource is the HTTP method and path from a restricted operation that returns Personally Identifiable Information (PII), plus a dataElements value that indicates the type of PII requested. See the Tokens API Use Case Guide for a list of restricted operations. Use the RDT returned here as the access token in subsequent calls to the corresponding restricted operations.
        **Usage Plans:**
        | Plan type | Rate (requests per second) | Burst |
        | ---- | ---- | ---- |
        |Default| 1 | 10 |
        |Selling partner specific| Variable | Variable |
        The x-amzn-RateLimit-Limit response header returns the usage plan rate limits that were applied to the requested operation. Rate limits for some selling partners will vary from the default rate and burst shown in the table above. For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/tokens/2021-03-01/restrictedDataToken"
        params = {}
        response = self.request(
            path=url,
            method="POST",
            params=params,
            data=data.data,
        )
        response_type = {
            200: CreateRestrictedDataTokenResponse,
            400: ErrorList,
            401: ErrorList,
            403: ErrorList,
            404: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))



from .base import BaseClient as __BaseClient, convert_bool, BaseDictObject as __BaseDictObject
from typing import List as _List


class CreateUploadDestinationResponse(__BaseDictObject):
    """
    The response schema for the createUploadDestination operation.
    """

    def __init__(self, data):
        super().__init__(data)
        if "payload" in data:
            self.payload: UploadDestination = self._get_value(UploadDestination, "payload")
        else:
            self.payload: UploadDestination = None
        if "errors" in data:
            self.errors: ErrorList = self._get_value(ErrorList, "errors")
        else:
            self.errors: ErrorList = None


class UploadDestination(__BaseDictObject):
    """
    Information about an upload destination.
    """

    def __init__(self, data):
        super().__init__(data)
        if "uploadDestinationId" in data:
            self.uploadDestinationId: str = self._get_value(str, "uploadDestinationId")
        else:
            self.uploadDestinationId: str = None
        if "url" in data:
            self.url: str = self._get_value(str, "url")
        else:
            self.url: str = None
        if "headers" in data:
            self.headers: dict = self._get_value(dict, "headers")
        else:
            self.headers: dict = None


class Error(__BaseDictObject):
    """
    Error response returned when the request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__(data)
        if "code" in data:
            self.code: str = self._get_value(str, "code")
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = self._get_value(str, "message")
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = self._get_value(str, "details")
        else:
            self.details: str = None


class ErrorList(list, _List["Error"]):
    """
    A list of error responses returned when a request is unsuccessful.
    """

    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class Uploads20201101Client(__BaseClient):
    def createUploadDestinationForResource(
        self,
        resource: str,
        marketplaceIds: _List[str],
        contentMD5: str,
        contentType: str = None,
    ):
        """
                Creates an upload destination, returning the information required to upload a file to the destination and to programmatically access the file.
        **Usage Plan:**
        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | .1 | 5 |
        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        """
        url = f"/uploads/2020-11-01/uploadDestinations/{resource}"
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if contentMD5 is not None:
            params["contentMD5"] = contentMD5
        if contentType is not None:
            params["contentType"] = contentType
        response = self.request(
            path=url,
            method="POST",
            params=params,
        )
        response_type = {
            201: CreateUploadDestinationResponse,
            400: CreateUploadDestinationResponse,
            403: CreateUploadDestinationResponse,
            404: CreateUploadDestinationResponse,
            413: CreateUploadDestinationResponse,
            415: CreateUploadDestinationResponse,
            429: CreateUploadDestinationResponse,
            500: CreateUploadDestinationResponse,
            503: CreateUploadDestinationResponse,
        }.get(response.status_code, None)
        return None if response_type is None else response_type(self._get_response_json(response))
